
module Top(
    input   clkin,		//12M
    //output reg    LED,
    output  uart_txd,
    input   uart_rxd,
    output  jtag_clk,      // JTAG Test clock
    output  jtag_tms,      // JTAG Test Mode Select
    input   jtag_tdo,      // JTAG Serial Input Data
    output  jtag_tdi,      // JTAG Serial Output Data
    inout   I2C1_SCL,
    inout   I2C1_SDA,
    output  spi1_clk_o ,
    output  spi1_cs_n_o ,
    inout   spi1_miso_io,
    inout   spi1_mosi_io,

    inout   usb_dxp_io,
    inout   usb_dxn_io,
    input   usb_rxdp_i,
    input   usb_rxdn_i,
    output  usb_pullup_en_o,
	inout   usb_term_dp_io,
	inout   usb_term_dn_io
);


wire [1:0]  PHY_XCVRSELECT      ;
wire        PHY_TERMSELECT      ;
wire [1:0]  PHY_OPMODE          ;
wire [1:0]  PHY_LINESTATE       ;
wire        PHY_TXVALID         ;
wire        PHY_TXREADY         ;
wire        PHY_RXVALID         ;
wire        PHY_RXACTIVE        ;
wire        PHY_RXERROR         ;
wire [7:0]  PHY_DATAIN          ;
wire [7:0]  PHY_DATAOUT         ;
wire        PHY_CLKOUT          ;

wire [15:0]  DESCROM_RADDR       ;
wire [ 7:0]  DESCROM_RDAT        ;
wire [7:0]   desc_index;
wire [7:0]   desc_type;
wire [15:0]  DESC_DEV_ADDR       ;
wire [15:0]  DESC_DEV_LEN        ;
wire [15:0]  DESC_QUAL_ADDR      ;
wire [15:0]  DESC_QUAL_LEN       ;
wire [15:0]  DESC_FSCFG_ADDR     ;
wire [15:0]  DESC_FSCFG_LEN      ;
wire [15:0]  DESC_HSCFG_ADDR     ;
wire [15:0]  DESC_HSCFG_LEN      ;
wire [15:0]  DESC_OSCFG_ADDR     ;
wire [15:0]  DESC_STRLANG_ADDR   ;
wire [15:0]  DESC_STRVENDOR_ADDR ;
wire [15:0]  DESC_STRVENDOR_LEN  ;
wire [15:0]  DESC_STRPRODUCT_ADDR;
wire [15:0]  DESC_STRPRODUCT_LEN ;
wire [15:0]  DESC_STRSERIAL_ADDR ;
wire [15:0]  DESC_STRSERIAL_LEN  ;
wire         DESCROM_HAVE_STRINGS;
parameter    ENDPT_UART_CONFIG =4'h0;

//==============================================================
reg [7:0] rst_cnt = 0;

assign RESET_IN = rst_cnt<32;

always@(posedge PHY_CLKOUT) begin
    if (rst_cnt <= 32) begin
        rst_cnt <= rst_cnt + 8'd1;
    end
end

//======LED
/*
reg [31:0] cnt;
always@(posedge PHY_CLKOUT) begin
    if (RESET_IN) begin
        LED <= 1'b0;
        cnt <= 32'd0;
    end
    else begin
        if (cnt >= 32'd60000000) begin
            cnt <= 32'd0;
        end
        else begin
            cnt <= cnt + 32'd1;
        end
        if (cnt == 12'd30000000) begin
            LED <= ~LED;
        end
    end
end
*/
//==============================================================
wire lock_o;
wire fclk_480M /*synthesis syn_keep=1*/;
wire clk_48;

// --- usb ---
wire [7:0] usb_txdat;
reg  [11:0]usb_txdat_len;
wire usb_txcork;
wire usb_txpop ;
wire usb_txact ;
wire usb_txpktfin;
wire [7:0] usb_rxdat ;
wire usb_rxval ;
wire usb_rxact ;
wire usb_rxrdy ;
wire usb_rxpktval;
wire [3:0] endpt_sel;
wire [11:0] txdat_len;
wire       setup_active;
wire       setup_val   ;
wire [7:0] setup_data  ;
wire       endpt0_send ;
wire  [7:0]endpt0_dat  ;
wire       ep_usb_rxrdy;
wire       ep_usb_txcork;
wire [11:0]ep_usb_txlen;
wire [7:0] ep_usb_txdat;

//==============================================================
//======uart 1-7
wire uart1_en, uart2_en, uart3_en, uart4_en;
wire uart5_en, uart6_en, uart7_en;

// --- uart 1 ---
wire [31:0]uart1_s_dte_rate;
wire [7:0] uart1_s_char_format;
wire [7:0] uart1_s_parity_type;
wire [7:0] uart1_s_data_bits;

wire [7:0] uart1_tx_data    ;
wire       uart1_tx_data_val;
wire       uart1_tx_busy    ;
wire [7:0] uart1_rx_data    ;
wire       uart1_rx_data_val;

// --- uart 2 ---
wire [31:0]uart2_s_dte_rate;
wire [7:0] uart2_s_char_format;
wire [7:0] uart2_s_parity_type;
wire [7:0] uart2_s_data_bits;

wire [7:0] uart2_tx_data    ;
wire       uart2_tx_data_val;
wire       uart2_tx_busy    ;
wire [7:0] uart2_rx_data    ;
wire       uart2_rx_data_val;

// --- uart 3 ---
wire [31:0]uart3_s_dte_rate;
wire [7:0] uart3_s_char_format;
wire [7:0] uart3_s_parity_type;
wire [7:0] uart3_s_data_bits;

wire [7:0] uart3_tx_data    ;
wire       uart3_tx_data_val;
wire       uart3_tx_busy    ;
wire [7:0] uart3_rx_data    ;
wire       uart3_rx_data_val;

// --- uart 4 ---
wire [31:0]uart4_s_dte_rate;
wire [7:0] uart4_s_char_format;
wire [7:0] uart4_s_parity_type;
wire [7:0] uart4_s_data_bits;

wire [7:0] uart4_tx_data    ;
wire       uart4_tx_data_val;
wire       uart4_tx_busy    ;
wire [7:0] uart4_rx_data    ;
wire       uart4_rx_data_val;
wire [11:0]txdat_len_uart_config;

// mpb
wire [7:0] mpb_tx_data    ;
wire       mpb_tx_data_val;
wire [7:0] mpb_rx_data    ;
wire       mpb_rx_data_val;

wire       usb_data_in_rdy;
wire       jtag_tck;

wire        i2c1_tx_dval;
wire [7:0]  i2c1_tx_data;
wire        i2c1_rx_dval;
wire [7:0]  i2c1_rx_data;

wire        spi1_tx_dval;
wire [7:0]  spi1_tx_data;
wire        spi1_rx_dval;
wire [7:0]  spi1_rx_data;


//==============================================================
//======PLL 
    Gowin_rPLL u_pll(
        .clkout (fclk_480M ), //output clkout
        .clkoutd(PHY_CLKOUT), //output clkout
        .lock   (lock_o    ), //output lock
        .clkin  (clkin    )  //input clkin
    );

    rpll_peripheral rpll_peripheral(
        .clkout(clk_48), //output clkout
        .clkin(clkin   ) //input clkin
    );

//==============================================================
//======UART_config
usb_uart_config u_usb_uart_config
(
     .PHY_CLKOUT         (PHY_CLKOUT           ) // clock
    ,.RESET_IN           (usb_busreset         )//reset RESET_IN
    ,.setup_active       (setup_active         )
    ,.endpt_sel          (endpt_sel            )
    ,.usb_rxval          (usb_rxval            ) 
    ,.usb_rxact          (usb_rxact            ) 
    ,.usb_rxdat          (usb_rxdat            )
    ,.usb_txact          (usb_txact            )
    ,.usb_txpop          (usb_txpop            )
    ,.txdat_len_o        (txdat_len_uart_config)

    ,.jtag_tck           (jtag_tck            )

    ,.uart_endpt0_dat_o (endpt0_dat           )
    ,.uart_endpt0_send_o(endpt0_send          ) 

    ,.uart1_en_o         (uart1_en             )
    ,.uart1_BAUD_RATE_o  (uart1_s_dte_rate     ) 
    ,.uart1_PARITY_BIT_o (uart1_s_parity_type  ) 
    ,.uart1_STOP_BIT_o   (uart1_s_char_format  ) 
    ,.uart1_DATA_BITS_o  (uart1_s_data_bits    ) 



);
defparam u_usb_uart_config.ENDPT_UART_CONFIG = ENDPT_UART_CONFIG;


//==============================================================
//======USB Controller 
assign usb_rxrdy = ep_usb_rxrdy;
assign usb_txcork =(endpt_sel ==ENDPT_UART_CONFIG) ? 1'b0 : ep_usb_txcork;
assign usb_txdat = (endpt_sel==ENDPT_UART_CONFIG) ? endpt0_dat : ep_usb_txdat;
assign txdat_len = (endpt_sel ==ENDPT_UART_CONFIG) ? txdat_len_uart_config : ep_usb_txlen;

localparam UART1_TX_PACKET_MAX    = 12'd512;
localparam MPB_TX_PACKET_MAX      = 12'd512;

usb_fifo usb_fifo
(
     .i_clk         (PHY_CLKOUT   )//clock
    ,.i_reset       (usb_busreset )//reset
    //,.i_reset       (RESET_IN     )//reset
    ,.i_usb_endpt   (endpt_sel    )//
    ,.i_usb_rxact   (usb_rxact    )//
    ,.i_usb_rxval   (usb_rxval    )//
    ,.i_usb_rxpktval(usb_rxpktval )//
    ,.i_usb_rxdat   (usb_rxdat    )//
    ,.o_usb_rxrdy   (ep_usb_rxrdy )//
    ,.i_usb_txact   (usb_txact    )//
    ,.i_usb_txpop   (usb_txpop    )//
    ,.i_usb_txpktfin(usb_txpktfin )//
    ,.o_usb_txcork  (ep_usb_txcork)//
    ,.o_usb_txlen   (ep_usb_txlen )//
    ,.o_usb_txdat   (ep_usb_txdat )//

    //Endpoint 1
//    ,.i_ep1_tx_clk  (PHY_CLKOUT       )//
//    ,.i_ep1_tx_max  (UART1_TX_PACKET_MAX)//
//    ,.i_ep1_tx_dval (uart1_rx_data_val)//
//    ,.i_ep1_tx_data (uart1_rx_data    )//
//    ,.i_ep1_rx_clk  (PHY_CLKOUT       )//
//    ,.i_ep1_rx_rdy  (!uart1_tx_busy   )//
//    ,.o_ep1_rx_dval (uart1_tx_data_val)//
//    ,.o_ep1_rx_data (uart1_tx_data    )//

    //Endpoint 2
    ,.i_ep2_tx_clk  (PHY_CLKOUT       )//
    ,.i_ep2_tx_max  (UART1_TX_PACKET_MAX)//
    ,.i_ep2_tx_dval (uart1_rx_data_val)//
    ,.i_ep2_tx_data (uart1_rx_data    )//
    ,.i_ep2_rx_clk  (PHY_CLKOUT       )//
    ,.i_ep2_rx_rdy  (!uart1_tx_busy  )//
    ,.o_ep2_rx_dval (uart1_tx_data_val)//
    ,.o_ep2_rx_data (uart1_tx_data    )//

    //Endpoint 3
    ,.i_ep3_tx_clk  (PHY_CLKOUT       )//
    ,.i_ep3_tx_max  (MPB_TX_PACKET_MAX)//
    ,.i_ep3_tx_dval (mpb_rx_data_val)//
    ,.i_ep3_tx_data (mpb_rx_data    )//
    ,.i_ep3_rx_clk  (PHY_CLKOUT       )//
    ,.i_ep3_rx_rdy  (usb_data_in_rdy   )//
    ,.o_ep3_rx_dval (mpb_tx_data_val)//
    ,.o_ep3_rx_data (mpb_tx_data    )//

    //Endpoint 4
    ,.i_ep4_tx_clk  (clk_48      )//
    ,.i_ep4_tx_max  (MPB_TX_PACKET_MAX)//
    ,.i_ep4_tx_dval (spi1_rx_dval)//
    ,.i_ep4_tx_data (spi1_rx_data)//
    ,.i_ep4_rx_clk  (clk_48      )//
    ,.i_ep4_rx_rdy  (1'b1        )//
    ,.o_ep4_rx_dval (spi1_tx_dval)//
    ,.o_ep4_rx_data (spi1_tx_data)//

    //Endpoint 5
    ,.i_ep5_tx_clk  (clk_48      )//
    ,.i_ep5_tx_max  (MPB_TX_PACKET_MAX)//
    ,.i_ep5_tx_dval (i2c1_rx_dval)//
    ,.i_ep5_tx_data (i2c1_rx_data)//
    ,.i_ep5_rx_clk  (clk_48      )//
    ,.i_ep5_rx_rdy  (1'b1        )//
    ,.o_ep5_rx_dval (i2c1_tx_dval)//
    ,.o_ep5_rx_data (i2c1_tx_data)//
);


USB_Device_Controller_Top u_usb_device_controller_top (
     .clk_i                 (PHY_CLKOUT          )
    ,.reset_i               (RESET_IN            )
    ,.usbrst_o              (usb_busreset        )
    ,.highspeed_o           (usb_highspeed       )
    ,.suspend_o             (usb_suspend         )
    ,.online_o              (usb_online          )
    ,.txdat_i               (usb_txdat           )
    ,.txval_i               (endpt0_send&(endpt_sel==ENDPT_UART_CONFIG))//
    ,.txdat_len_i           (txdat_len           )
    ,.txcork_i              (usb_txcork          )
    ,.txpop_o               (usb_txpop           )
	,.txiso_pid_i           (4'b0011             )//DATA0
    ,.txact_o               (usb_txact           )
    ,.txpktfin_o            (usb_txpktfin        )
    ,.rxdat_o               (usb_rxdat           )
    ,.rxval_o               (usb_rxval           )
    ,.rxact_o               (usb_rxact           )
    ,.rxrdy_i               (usb_rxrdy           )
    ,.rxpktval_o            (usb_rxpktval        )
    ,.setup_o               (setup_active        )
    ,.endpt_o               (endpt_sel           )
    ,.sof_o                 (usb_sof             )
    ,.descrom_rdata_i       (DESCROM_RDAT        )
    ,.descrom_raddr_o       (DESCROM_RADDR       )
    ,.desc_index_o          (desc_index          )
    ,.desc_type_o           (desc_type           )
    ,.desc_dev_addr_i       (DESC_DEV_ADDR       )
    ,.desc_dev_len_i        (DESC_DEV_LEN        )
    ,.desc_qual_addr_i      (DESC_QUAL_ADDR      )
    ,.desc_qual_len_i       (DESC_QUAL_LEN       )
    ,.desc_fscfg_addr_i     (DESC_FSCFG_ADDR     )
    ,.desc_fscfg_len_i      (DESC_FSCFG_LEN      )
    ,.desc_hscfg_addr_i     (DESC_HSCFG_ADDR     )
    ,.desc_hscfg_len_i      (DESC_HSCFG_LEN      )
    ,.desc_oscfg_addr_i     (DESC_OSCFG_ADDR     )
    ,.desc_strlang_addr_i   (DESC_STRLANG_ADDR   )
    ,.desc_strvendor_addr_i (DESC_STRVENDOR_ADDR )
    ,.desc_strvendor_len_i  (DESC_STRVENDOR_LEN  )
    ,.desc_strproduct_addr_i(DESC_STRPRODUCT_ADDR)
    ,.desc_strproduct_len_i (DESC_STRPRODUCT_LEN )
    ,.desc_strserial_addr_i (DESC_STRSERIAL_ADDR )
    ,.desc_strserial_len_i  (DESC_STRSERIAL_LEN  )
    ,.desc_have_strings_i   (DESCROM_HAVE_STRINGS)
    ,.utmi_dataout_o        (PHY_DATAOUT         )
    ,.utmi_txvalid_o        (PHY_TXVALID         )
    ,.utmi_txready_i        (PHY_TXREADY         )
    ,.utmi_datain_i         (PHY_DATAIN          )
    ,.utmi_rxactive_i       (PHY_RXACTIVE        )
    ,.utmi_rxvalid_i        (PHY_RXVALID         )
    ,.utmi_rxerror_i        (PHY_RXERROR         )
    ,.utmi_linestate_i      (PHY_LINESTATE       )
    ,.utmi_opmode_o         (PHY_OPMODE          )
    ,.utmi_xcvrselect_o     (PHY_XCVRSELECT      )
    ,.utmi_termselect_o     (PHY_TERMSELECT      )
    ,.utmi_reset_o          (PHY_RESET           )
);

usb_desc
#(

     .VENDORID    (16'h33AA)//0403   08bb
    ,.PRODUCTID   (16'h0021)//h0160
//    ,.PRODUCTID   (16'h0160)//h0160
    ,.VERSIONBCD  (16'h0100)
    ,.HSSUPPORT   (1)
    ,.SELFPOWERED (0)
)
u_usb_desc (
     .CLK                    (PHY_CLKOUT          )
    ,.RESET                  (RESET_IN            )
    ,.i_descrom_raddr        (DESCROM_RADDR       )
    ,.o_descrom_rdat         (DESCROM_RDAT        )
    ,.i_desc_index_o         (desc_index          )
    ,.i_desc_type_o          (desc_type           )
    ,.o_desc_dev_addr        (DESC_DEV_ADDR       )
    ,.o_desc_dev_len         (DESC_DEV_LEN        )
    ,.o_desc_qual_addr       (DESC_QUAL_ADDR      )
    ,.o_desc_qual_len        (DESC_QUAL_LEN       )
    ,.o_desc_fscfg_addr      (DESC_FSCFG_ADDR     )
    ,.o_desc_fscfg_len       (DESC_FSCFG_LEN      )
    ,.o_desc_hscfg_addr      (DESC_HSCFG_ADDR     )
    ,.o_desc_hscfg_len       (DESC_HSCFG_LEN      )
    ,.o_desc_oscfg_addr      (DESC_OSCFG_ADDR     )
    ,.o_desc_strlang_addr    (DESC_STRLANG_ADDR   )
    ,.o_desc_strvendor_addr  (DESC_STRVENDOR_ADDR )
    ,.o_desc_strvendor_len   (DESC_STRVENDOR_LEN  )
    ,.o_desc_strproduct_addr (DESC_STRPRODUCT_ADDR)
    ,.o_desc_strproduct_len  (DESC_STRPRODUCT_LEN )
    ,.o_desc_strserial_addr  (DESC_STRSERIAL_ADDR )
    ,.o_desc_strserial_len   (DESC_STRSERIAL_LEN  )
    ,.o_descrom_have_strings (DESCROM_HAVE_STRINGS)
);

//==============================================================
//======USB SoftPHY 
    USB2_0_SoftPHY_Top u_USB_SoftPHY_Top
    (
         .clk_i            (PHY_CLKOUT     )
        ,.rst_i            (PHY_RESET      )
        ,.pll_locked_i     (lock_o         )

        ,.fclk_i           (fclk_480M      )
        ,.utmi_data_out_i  (PHY_DATAOUT    )
        ,.utmi_txvalid_i   (PHY_TXVALID    )
        ,.utmi_op_mode_i   (PHY_OPMODE     )
        ,.utmi_xcvrselect_i(PHY_XCVRSELECT )
        ,.utmi_termselect_i(PHY_TERMSELECT )
        ,.utmi_data_in_o   (PHY_DATAIN     )
        ,.utmi_txready_o   (PHY_TXREADY    )
        ,.utmi_rxvalid_o   (PHY_RXVALID    )
        ,.utmi_rxactive_o  (PHY_RXACTIVE   )
        ,.utmi_rxerror_o   (PHY_RXERROR    )
        ,.utmi_linestate_o (PHY_LINESTATE  )
        ,.usb_dxp_io       (usb_dxp_io     )
        ,.usb_dxn_io       (usb_dxn_io     )
        ,.usb_rxdp_i       (usb_rxdp_i     )
        ,.usb_rxdn_i       (usb_rxdn_i     )
        ,.usb_pullup_en_o  (usb_pullup_en_o)
        ,.usb_term_dp_io   (usb_term_dp_io )
        ,.usb_term_dn_io   (usb_term_dn_io )
    );

    USB_to_Multi_Serial_Top USB_to_Multi_Serial(
        .phy_clk        (PHY_CLKOUT), //input phy_clk
        .rst_n          (usb_busreset), //input rst_n
        .UART_TXD       (uart_txd), //output UART_TXD
        .UART_RXD       (uart_rxd), //input UART_RXD
        .UART_RTS       (), //output UART_RTS
        .UART_CTS       (1'b0), //input UART_CTS
        .BAUD_RATE      (uart1_s_dte_rate    ), //input [31:0] BAUD_RATE    0001C200
        .PARITY_BIT     (uart1_s_parity_type ), //input [7:0] PARITY_BIT    00
        .STOP_BIT       (uart1_s_char_format ), //input [7:0] STOP_BIT      00
        .DATA_BITS      (uart1_s_data_bits   ), //input [7:0] DATA_BITS     08
        .TX_DATA        (uart1_tx_data       ), //input [15:0] TX_DATA
        .TX_DATA_VAL    (uart1_tx_data_val ), //input TX_DATA_VAL
        .TX_BUSY        (uart1_tx_busy     ), //output TX_BUSY
        .RX_DATA        (uart1_rx_data     ), //output [15:0] RX_DATA
        .RX_DATA_VAL    (uart1_rx_data_val ), //output RX_DATA_VAL
    
        .usb_jtag_tck       (jtag_tck           ), //input usb_jtag_tck
        .usb_jtag_rx_dval   (mpb_tx_data_val    )  , //input usb_jtag_rx_dval
        .usb_jtag_rx_data   (mpb_tx_data        )  , //input [7:0] usb_jtag_rx_data
        .usb_jtag_in_rdy    (usb_data_in_rdy    ), //output usb_jtag_in_rdy
        .usb_jtag_tx_dval   (mpb_rx_data_val    ), //output usb_jtag_tx_dval
        .usb_jtag_tx_data   (mpb_rx_data        ), //output [7:0] usb_jtag_tx_data
        .jtag_clk(jtag_clk ), //output jtag_clk
        .jtag_tms(jtag_tms    ), //output jtag_tms
        .jtag_tdo(jtag_tdo    ), //input jtag_tdo
        .jtag_tdi(jtag_tdi    ), //output jtag_tdi
    
        .usb_spi_clk        (clk_48       ), //input usb_spi_clk
        .usb_spi_rx_dval    (spi1_tx_dval ), //input usb_spi_rx_dval
        .usb_spi_rx_data    (spi1_tx_data ), //input [7:0] usb_spi_rx_data
        .usb_spi_tx_dval    (spi1_rx_dval ), //output usb_spi_tx_dval
        .usb_spi_tx_data    (spi1_rx_data ), //output [7:0] usb_spi_tx_data
        .spi_clk_o  (spi1_clk_o  ), //output spi_clk_o
        .spi_cs_n_o (spi1_cs_n_o ), //output spi_cs_n_o
        .spi_miso_io(spi1_miso_io), //inout spi_miso_io
        .spi_mosi_io(spi1_mosi_io), //inout spi_mosi_io
    
        .usb_i2c_clk    (clk_48           ), //input usb_i2c_clk
        .usb_i2c_rx_dval(i2c1_tx_dval ), //input usb_i2c_rx_dval
        .usb_i2c_rx_data(i2c1_tx_data ), //input [7:0] usb_i2c_rx_data
        .usb_i2c_tx_dval(i2c1_rx_dval ), //output usb_i2c_tx_dval
        .usb_i2c_tx_data(i2c1_rx_data ), //output [7:0] usb_i2c_tx_data
        .I2C1_SCL(I2C1_SCL), //inout I2C1_SCL
        .I2C1_SDA(I2C1_SDA), //inout I2C1_SDA
        .status_o() //output status_o
    );

endmodule



