
//===========================================
module usb_desc #(
        // Vendor ID to report in device descriptor.
        parameter VENDORID = 16'h33AA,//16'h0403,//fb9a;//16'hfb9a;//16'h08bb;
        // Product ID to report in device descriptor.
        parameter PRODUCTID = 16'h0021,//16'h6010,//fb9a;//16'hfb9a;//16'h27c6;
        // Product version to report in device descriptor.
        parameter VERSIONBCD = 16'h0100,//16'h0100;
        // Optional description of manufacturer (max 126 characters).
        parameter VENDORSTR = "Gowinsemi",
        parameter VENDORSTR_LEN = 9,//66;
        // Optional description of product (max 126 characters).
//        parameter PRODUCTSTR = "Gowin Multi Channels",
//        parameter PRODUCTSTR_LEN = 20,//34;
        parameter PRODUCTSTR = "Gowin Serial Port",
        parameter PRODUCTSTR_LEN = 17,//34;

        parameter STR3 = "Gowin Serial Converter",
        parameter STR3_LEN = 22,
        parameter STR4 = "Gowin Serial Ch1",
        parameter STR4_LEN = 16,
        parameter STR5 = "Gowin Serial Ch2",
        parameter STR5_LEN = 16,
        parameter STR6 = "Gowin Serial Ch3",
        parameter STR6_LEN = 16,

        // Optional product serial number (max 126 characters).
        parameter SERIALSTR = "Blank string",
        parameter SERIALSTR_LEN = 0,
        // Support high speed mode.
        parameter HSSUPPORT = 0,
        // Set to true if the device never draws power from the USB bus.
        parameter SELFPOWERED = 0
)
(

        input        CLK                     ,
        input        RESET                   ,
        input  [15:0] i_descrom_raddr        ,
        output [ 7:0] o_descrom_rdat         ,
        input  [ 7:0] i_desc_index_o         ,
        input  [ 7:0] i_desc_type_o          ,
        output [15:0] o_desc_dev_addr        ,
        output [15:0] o_desc_dev_len         ,
        output [15:0] o_desc_qual_addr       ,
        output [15:0] o_desc_qual_len        ,
        output [15:0] o_desc_fscfg_addr      ,
        output [15:0] o_desc_fscfg_len       ,
        output [15:0] o_desc_hscfg_addr      ,
        output [15:0] o_desc_hscfg_len       ,
        output [15:0] o_desc_oscfg_addr      ,
        output [15:0] o_desc_strlang_addr    ,
        output [15:0] o_desc_strvendor_addr  ,
        output [15:0] o_desc_strvendor_len   ,
        output [15:0] o_desc_strproduct_addr ,
        output [15:0] o_desc_strproduct_len  ,
        output [15:0] o_desc_strserial_addr  ,
        output [15:0] o_desc_strserial_len   ,
        output       o_descrom_have_strings
);

    // Truncate descriptor data to keep only the necessary pieces;
    // either just the full-speed stuff, || full-speed plus high-speed,
    // || full-speed plus high-speed plus string descriptors.

    localparam  DESC_INTERFACE_NUM           = 2;
    // Descriptor ROM
    //   addr   0 ..  17 : device descriptor
    //   addr  20 ..  29 : device qualifier
    //   addr  32 ..  98 : full speed configuration descriptor 
    //   addr 112 .. 178 : high speed configuration descriptor
    //   addr 179 :        other_speed_configuration hack
    //   addr 192 .. 195 : string descriptor 0 = supported languages
    //   addr 196 ..     : 3 string descriptors: vendor, product, serial
    localparam  DESC_DEV_ADDR         = 0;
    localparam  DESC_DEV_LEN          = 18;
    localparam  DESC_QUAL_ADDR        = 20;
    localparam  DESC_QUAL_LEN         = 10;
    localparam  DESC_FSCFG_ADDR       = 32;
    localparam  DESC_FSCFG_LEN_7CH    = 90; //7:219;  5:131  
    localparam  DESC_FSCFG_LEN        = 90; //7uart:219; 4uart:129;    CH1:39  CH2:69  CH3:85 CH4:108 CH5:131
    localparam  DESC_HSCFG_ADDR       = DESC_FSCFG_ADDR;//DESC_FSCFG_ADDR + DESC_FSCFG_LEN;//222;222;//
    localparam  DESC_HSCFG_LEN        = DESC_FSCFG_LEN;
//    localparam  DESC_OSCFG_ADDR       = DESC_HSCFG_ADDR + DESC_HSCFG_LEN;// 179;//289;289;//
    localparam  DESC_OSCFG_ADDR       = DESC_HSCFG_ADDR + DESC_FSCFG_LEN_7CH;// 179;//289;289;//
    localparam  DESC_OSCFG_LEN        = 41 ;//67; 67; //
//    localparam  DESC_STRLANG_ADDR     = DESC_FSCFG_ADDR + DESC_FSCFG_LEN;// 192;//330;330;//
    localparam  DESC_STRLANG_ADDR     = DESC_FSCFG_ADDR + DESC_FSCFG_LEN_7CH;// 192;//330;330;//
    localparam  DESC_STRVENDOR_ADDR   = DESC_STRLANG_ADDR + 4;// 196;//334;334;//
    localparam  DESC_STRVENDOR_LEN    = 2 + 2*VENDORSTR_LEN;// 196;//334;334;//
    localparam  DESC_STRPRODUCT_ADDR  = DESC_STRVENDOR_ADDR + DESC_STRVENDOR_LEN;
    localparam  DESC_STRPRODUCT_LEN   = 2 + 2*PRODUCTSTR_LEN;

    localparam  DESC_STR3_ADDR        = DESC_STRPRODUCT_ADDR + DESC_STRPRODUCT_LEN;
    localparam  DESC_STR3_LEN         = 2 + 2*STR3_LEN;

    localparam  DESC_STR4_ADDR        = DESC_STR3_ADDR + DESC_STR3_LEN;
    localparam  DESC_STR4_LEN         = 2 + 2*STR3_LEN;

    localparam  DESC_STR5_ADDR        = DESC_STR4_ADDR + DESC_STR4_LEN;
    localparam  DESC_STR5_LEN         = 2 + 2*STR4_LEN;

    localparam  DESC_STR6_ADDR        = DESC_STR5_ADDR + DESC_STR5_LEN;
    localparam  DESC_STR6_LEN         = 2 + 2*STR5_LEN;

    localparam  DESC_STRSERIAL_ADDR   = DESC_STR3_ADDR + DESC_STR3_LEN;
    localparam  DESC_STRSERIAL_LEN    = 2 + 2*SERIALSTR_LEN;
    localparam  DESC_END_ADDR         = DESC_STRSERIAL_ADDR + DESC_STRSERIAL_LEN;
 
    assign  o_desc_dev_addr        = DESC_DEV_ADDR        ;
    assign  o_desc_dev_len         = DESC_DEV_LEN         ;
    assign  o_desc_qual_addr       = DESC_QUAL_ADDR       ;
    assign  o_desc_qual_len        = DESC_QUAL_LEN        ;
    assign  o_desc_fscfg_addr      = DESC_FSCFG_ADDR      ;
    assign  o_desc_fscfg_len       = DESC_FSCFG_LEN       ;
    assign  o_desc_hscfg_addr      = DESC_HSCFG_ADDR      ;
    assign  o_desc_hscfg_len       = DESC_HSCFG_LEN       ;
    assign  o_desc_oscfg_addr      = DESC_OSCFG_ADDR      ;
    assign  o_desc_strlang_addr    = DESC_STRLANG_ADDR    ;
    assign  o_desc_strvendor_addr  = DESC_STRVENDOR_ADDR  ;
    assign  o_desc_strvendor_len   = DESC_STRVENDOR_LEN   ;
    assign  o_desc_strproduct_addr = DESC_STRPRODUCT_ADDR ;
    assign  o_desc_strproduct_len  = DESC_STRPRODUCT_LEN  ;

//    assign  o_desc_strserial_addr  = DESC_STRSERIAL_ADDR  ;
//    assign  o_desc_strserial_len   = DESC_STRSERIAL_LEN   ;
    assign  o_desc_strserial_addr  = (i_desc_index_o == 3) ? DESC_STR3_ADDR : DESC_STRPRODUCT_ADDR;
//                                     (i_desc_index_o == 4) ? DESC_STR4_ADDR :
//                                     (i_desc_index_o == 5) ? DESC_STR5_ADDR :
//                                     (i_desc_index_o == 6) ? DESC_STR6_ADDR : DESC_STRPRODUCT_ADDR;
    assign  o_desc_strserial_len   = (i_desc_index_o == 3) ? DESC_STR3_LEN : DESC_STRPRODUCT_LEN;
//                                     (i_desc_index_o == 4) ? DESC_STR4_LEN :
//                                     (i_desc_index_o == 5) ? DESC_STR5_LEN :
//                                     (i_desc_index_o == 6) ? DESC_STR6_LEN : DESC_STRPRODUCT_LEN;

    // Truncate descriptor data to keep only the necessary pieces;
    // either just the full-speed stuff, || full-speed plus high-speed,
    // || full-speed plus high-speed plus string descriptors.
    
    localparam descrom_have_strings = (VENDORSTR_LEN > 0 || PRODUCTSTR_LEN > 0 || SERIALSTR_LEN > 0);
    localparam descrom_len = (HSSUPPORT || descrom_have_strings)?((descrom_have_strings)? DESC_END_ADDR : DESC_OSCFG_ADDR + DESC_OSCFG_LEN) : DESC_FSCFG_ADDR + DESC_FSCFG_LEN;
    assign o_descrom_have_strings = descrom_have_strings;
    reg [7:0] descrom [0 : descrom_len-1];
    integer i;
    integer z;
    
    always @(posedge CLK or posedge RESET)
      if(RESET) begin
        // 18 bytes device descriptor
        descrom[0]  <= 8'h12;// bLength = 18 bytes
        descrom[1]  <= 8'h01;// bDescriptorType = device descriptor
        descrom[2]  <= (HSSUPPORT)? 8'h00 :8'h10;// bcdUSB = 1.10 || 2.00
        descrom[3]  <= (HSSUPPORT)? 8'h02 :8'h01;
        descrom[4]  <= 8'h00;// 00: 02:bDeviceClass = Communication Device Class
        descrom[5]  <= 8'h00;// bDeviceSubClass = none
        descrom[6]  <= 8'h00;// bDeviceProtocol = none
        descrom[7]  <= 8'h40;// 08: 40:bMaxPacketSize0 = 64 bytes
        descrom[8]  <= VENDORID[7 : 0];// idVendor
        descrom[9]  <= VENDORID[15 :8];
        descrom[10] <= PRODUCTID[7 :0];// idProduct
        descrom[11] <= PRODUCTID[15 :8];
        descrom[12] <= VERSIONBCD[7 : 0];// bcdDevice
        descrom[13] <= VERSIONBCD[15 : 8];
        descrom[14] <= (VENDORSTR_LEN > 0)?  8'h01: 8'h00;// iManufacturer
        descrom[15] <= (PRODUCTSTR_LEN > 0)? 8'h02: 8'h00;// iProduct
        descrom[16] <= (SERIALSTR_LEN > 0)?  8'h03: 8'h00;// iSerialNumber
        descrom[17] <= 8'h01;                  // bNumConfigurations = 1
        // 2 bytes padding
        descrom[18] <= 8'h00;
        descrom[19] <= 8'h00;
        // 10 bytes device qualifier
        descrom[20 + 0] <= 8'h0a;// bLength = 10 bytes
        descrom[20 + 1] <= 8'h06;// bDescriptorType = device qualifier
        descrom[20 + 2] <= 8'h00;
        descrom[20 + 3] <= 8'h02;// bcdUSB = 2.0
        descrom[20 + 4] <= 8'h02;// bDeviceClass = Communication Device Class
        descrom[20 + 5] <= 8'h00;// bDeviceSubClass = none
        descrom[20 + 6] <= 8'h00;// bDeviceProtocol = none
        descrom[20 + 7] <= 8'h00;// bMaxPacketSize0 = 512 bytes
        descrom[20 + 8] <= 8'h00;// bNumConfigurations = 1
        descrom[20 + 9] <= 8'h00;// bReserved
         // 2 bytes padding
        descrom[20 + 10] <= 8'h00;
        descrom[20 + 11] <= 8'h00;
        // 190 bytes full-speed configuration descriptor
        // 9 bytes configuration header
        descrom[DESC_FSCFG_ADDR + 0] <= 8'h09;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 1] <= 8'h02;// bDescriptorType = configuration descriptor
        descrom[DESC_FSCFG_ADDR + 2] <= DESC_FSCFG_LEN[7:0];// 39 bytes
        descrom[DESC_FSCFG_ADDR + 3] <= DESC_FSCFG_LEN[15:8];// wTotalLength = 55 bytes
        descrom[DESC_FSCFG_ADDR + 4] <= DESC_INTERFACE_NUM;// bNumInterfaces = 4
        descrom[DESC_FSCFG_ADDR + 5] <= 8'h01;// bConfigurationValue = 1
        descrom[DESC_FSCFG_ADDR + 6] <= 8'h00;// iConfiguration = none
        descrom[DESC_FSCFG_ADDR + 7] <= (SELFPOWERED)? 8'hc0 : 8'h80; // bmAttributes
        descrom[DESC_FSCFG_ADDR + 8] <= 8'hFA;// bMaxPower = 500 mA
		
        //---------------- Interface Descriptor -------------------
        //------------Serial COM 0 uart1 interface------------------ 
        // bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02
        descrom[DESC_FSCFG_ADDR + 9 ] <= 8'h09;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 10] <= 8'h04;// bDescriptorType = interface descriptor
        descrom[DESC_FSCFG_ADDR + 11] <= 8'h00;// bInterfaceNumber = 0
        descrom[DESC_FSCFG_ADDR + 12] <= 8'h00;// bAlternateSetting = 0
        descrom[DESC_FSCFG_ADDR + 13] <= 8'h03;// bNumEndpoints = 3
        descrom[DESC_FSCFG_ADDR + 14] <= 8'h02;// bInterfaceClass = CDC
        descrom[DESC_FSCFG_ADDR + 15] <= 8'h01;// bInterfaceSubClass = none
        descrom[DESC_FSCFG_ADDR + 16] <= 8'h00;// bInterafceProtocol = none
        descrom[DESC_FSCFG_ADDR + 17] <= 8'h02;// iInterface = none
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 18] <= 8'h07;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 19] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 20] <= 8'h82;// bEndpointAddress = INPUT 2
        descrom[DESC_FSCFG_ADDR + 21] <= 8'h02;// bmAttributes = TransferType=Isochronous  SyncType=Adaptive  EndpointType=Data
        descrom[DESC_FSCFG_ADDR + 22] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 23] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 24] <= 8'h00;// bInterval = 0 ms
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 25] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 26] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 27] <= 8'h02;// bEndpointAddress = OUTPUT 2
        descrom[DESC_FSCFG_ADDR + 28] <= 8'h02;// TransferType = Bulk
        descrom[DESC_FSCFG_ADDR + 29] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 30] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 31] <= 8'h00;// bInterval = 0 ms
		//----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 32 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 32 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 32 + 2] <= 8'h8A;// bEndpointAddress = INPUT A
        descrom[DESC_FSCFG_ADDR + 32 + 3] <= 8'h03;// TransferType = Interrupr
        descrom[DESC_FSCFG_ADDR + 32 + 4] <= 8'h08;
        descrom[DESC_FSCFG_ADDR + 32 + 5] <= 8'h00;// wMaxPacketSize = 8 bytes
        descrom[DESC_FSCFG_ADDR + 32 + 6] <= 8'h08;// bInterval = 0 ms
		
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        // bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02
        descrom[DESC_FSCFG_ADDR + 39 + 0] <= 8'h09;// bLength = 9 bytes                                                               
        descrom[DESC_FSCFG_ADDR + 39 + 1] <= 8'h04;// bDescriptorType = interface descriptor                                          
        descrom[DESC_FSCFG_ADDR + 39 + 2] <= 8'h01;// bInterfaceNumber = 1                                                            
        descrom[DESC_FSCFG_ADDR + 39 + 3] <= 8'h00;// bAlternateSetting = 0                                                           
        descrom[DESC_FSCFG_ADDR + 39 + 4] <= 8'h06;// bNumEndpoints = 6                                                               
        descrom[DESC_FSCFG_ADDR + 39 + 5] <= 8'hFF;// bInterfaceClass = CDC                                                           
        descrom[DESC_FSCFG_ADDR + 39 + 6] <= 8'hFF;// bInterfaceSubClass = none                                                       
        descrom[DESC_FSCFG_ADDR + 39 + 7] <= 8'hFF;// bInterafceProtocol = none                                                      
        descrom[DESC_FSCFG_ADDR + 39 + 8] <= 8'h03;// iInterface = none    
        //------------Channel 1 JTAG interface------------------                                                           
        //----------------- Endpoint Descriptor -----------------                                                                     
        descrom[DESC_FSCFG_ADDR + 48 + 0] <= 8'h07;// bLength = 9 bytes                                                               
        descrom[DESC_FSCFG_ADDR + 48 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor                                           
        descrom[DESC_FSCFG_ADDR + 48 + 2] <= 8'h83;// bEndpointAddress = INPUT 3                                                      
        descrom[DESC_FSCFG_ADDR + 48 + 3] <= 8'h02;// bmAttributes = TransferType=Isochronous  SyncType=Adaptive  EndpointType=Data   
        descrom[DESC_FSCFG_ADDR + 48 + 4] <= 8'h00;                                                                                   
        descrom[DESC_FSCFG_ADDR + 48 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                                                      
        descrom[DESC_FSCFG_ADDR + 48 + 6] <= 8'h00;// bInterval = 0 ms                                                                
        //----------------- Endpoint Descriptor -----------------                                                                     
        descrom[DESC_FSCFG_ADDR + 55 + 0] <= 8'h07;// bLength = 7 bytes                                                               
        descrom[DESC_FSCFG_ADDR + 55 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor                                           
        descrom[DESC_FSCFG_ADDR + 55 + 2] <= 8'h03;// bEndpointAddress = OUTPUT 3                                                     
        descrom[DESC_FSCFG_ADDR + 55 + 3] <= 8'h02;// TransferType = Bulk                                                             
        descrom[DESC_FSCFG_ADDR + 55 + 4] <= 8'h00;                                                                                   
        descrom[DESC_FSCFG_ADDR + 55 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                                                      
        descrom[DESC_FSCFG_ADDR + 55 + 6] <= 8'h00;// bInterval = 0 ms                                                                

        //------------Channel 2 SPI interface------------------                                      
        //----------------- Endpoint Descriptor -----------------                                    
        descrom[DESC_FSCFG_ADDR + 62 + 0] <= 8'h07;// bLength = 9 bytes                              
        descrom[DESC_FSCFG_ADDR + 62 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor          
        descrom[DESC_FSCFG_ADDR + 62 + 2] <= 8'h84;// bEndpointAddress = INPUT 4                     
        descrom[DESC_FSCFG_ADDR + 62 + 3] <= 8'h02;// TransferType = Bulk                            
        descrom[DESC_FSCFG_ADDR + 62 + 4] <= 8'h00;                                                  
        descrom[DESC_FSCFG_ADDR + 62 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                     
        descrom[DESC_FSCFG_ADDR + 62 + 6] <= 8'h00;// bInterval = 0 ms                               
        //----------------- Endpoint Descriptor -----------------                                    
        descrom[DESC_FSCFG_ADDR + 69 + 0] <= 8'h07;// bLength = 7 bytes                              
        descrom[DESC_FSCFG_ADDR + 69 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor          
        descrom[DESC_FSCFG_ADDR + 69 + 2] <= 8'h04;// bEndpointAddress = OUTPUT 4                    
        descrom[DESC_FSCFG_ADDR + 69 + 3] <= 8'h02;// TransferType = Bulk                            
        descrom[DESC_FSCFG_ADDR + 69 + 4] <= 8'h00;                                                  
        descrom[DESC_FSCFG_ADDR + 69 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                     
        descrom[DESC_FSCFG_ADDR + 69 + 6] <= 8'h00;// bInterval = 0 ms                               
                                                                                                     
        //------------Channel 3 I2C interface------------------                                      
        //----------------- Endpoint Descriptor -----------------                                    
        descrom[DESC_FSCFG_ADDR + 76 + 0] <= 8'h07;// bLength = 9 bytes                              
        descrom[DESC_FSCFG_ADDR + 76 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor          
        descrom[DESC_FSCFG_ADDR + 76 + 2] <= 8'h85;// bEndpointAddress = INPUT 5                     
        descrom[DESC_FSCFG_ADDR + 76 + 3] <= 8'h02;// TransferType = Bulk                            
        descrom[DESC_FSCFG_ADDR + 76 + 4] <= 8'h00;                                                  
        descrom[DESC_FSCFG_ADDR + 76 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                     
        descrom[DESC_FSCFG_ADDR + 76 + 6] <= 8'h00;// bInterval = 0 ms                               
        //----------------- Endpoint Descriptor -----------------                                    
        descrom[DESC_FSCFG_ADDR + 83 + 0] <= 8'h07;// bLength = 7 bytes                              
        descrom[DESC_FSCFG_ADDR + 83 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor          
        descrom[DESC_FSCFG_ADDR + 83 + 2] <= 8'h05;// bEndpointAddress = OUTPUT 5                    
        descrom[DESC_FSCFG_ADDR + 83 + 3] <= 8'h02;// TransferType = Bulk                            
        descrom[DESC_FSCFG_ADDR + 83 + 4] <= 8'h00;                                                  
        descrom[DESC_FSCFG_ADDR + 83 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                     
        descrom[DESC_FSCFG_ADDR + 83 + 6] <= 8'h00;// bInterval = 0 ms                               
 

/*
        //------------Channel 1 JTAG interface------------------
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        // bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02
        descrom[DESC_FSCFG_ADDR + 39 + 0] <= 8'h09;// bLength = 9 bytes                                                               
        descrom[DESC_FSCFG_ADDR + 39 + 1] <= 8'h04;// bDescriptorType = interface descriptor                                          
        descrom[DESC_FSCFG_ADDR + 39 + 2] <= 8'h01;// bInterfaceNumber = 1                                                            
        descrom[DESC_FSCFG_ADDR + 39 + 3] <= 8'h00;// bAlternateSetting = 0                                                           
        descrom[DESC_FSCFG_ADDR + 39 + 4] <= 8'h02;// bNumEndpoints = 2                                                               
        descrom[DESC_FSCFG_ADDR + 39 + 5] <= 8'hFF;// bInterfaceClass = CDC                                                           
        descrom[DESC_FSCFG_ADDR + 39 + 6] <= 8'hFF;// bInterfaceSubClass = none                                                       
        descrom[DESC_FSCFG_ADDR + 39 + 7] <= 8'hFF;// bInterafceProtocol = none                                                      
        descrom[DESC_FSCFG_ADDR + 39 + 8] <= 8'h04;// iInterface = none                                                               
        //----------------- Endpoint Descriptor -----------------                                                                     
        descrom[DESC_FSCFG_ADDR + 48 + 0] <= 8'h07;// bLength = 9 bytes                                                               
        descrom[DESC_FSCFG_ADDR + 48 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor                                           
        descrom[DESC_FSCFG_ADDR + 48 + 2] <= 8'h83;// bEndpointAddress = INPUT 3                                                      
        descrom[DESC_FSCFG_ADDR + 48 + 3] <= 8'h02;// bmAttributes = TransferType=Isochronous  SyncType=Adaptive  EndpointType=Data   
        descrom[DESC_FSCFG_ADDR + 48 + 4] <= 8'h00;                                                                                   
        descrom[DESC_FSCFG_ADDR + 48 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                                                      
        descrom[DESC_FSCFG_ADDR + 48 + 6] <= 8'h00;// bInterval = 0 ms                                                                
        //----------------- Endpoint Descriptor -----------------                                                                     
        descrom[DESC_FSCFG_ADDR + 55 + 0] <= 8'h07;// bLength = 7 bytes                                                               
        descrom[DESC_FSCFG_ADDR + 55 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor                                           
        descrom[DESC_FSCFG_ADDR + 55 + 2] <= 8'h03;// bEndpointAddress = OUTPUT 3                                                     
        descrom[DESC_FSCFG_ADDR + 55 + 3] <= 8'h02;// TransferType = Bulk                                                             
        descrom[DESC_FSCFG_ADDR + 55 + 4] <= 8'h00;                                                                                   
        descrom[DESC_FSCFG_ADDR + 55 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                                                      
        descrom[DESC_FSCFG_ADDR + 55 + 6] <= 8'h00;// bInterval = 0 ms                                                                

        //------------Channel 2 SPI interface------------------
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        // bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02
        descrom[DESC_FSCFG_ADDR + 62 + 0] <= 8'h09;// bLength = 9 bytes                                                            
        descrom[DESC_FSCFG_ADDR + 62 + 1] <= 8'h04;// bDescriptorType = interface descriptor                                       
        descrom[DESC_FSCFG_ADDR + 62 + 2] <= 8'h02;// bInterfaceNumber = 2                                                         
        descrom[DESC_FSCFG_ADDR + 62 + 3] <= 8'h00;// bAlternateSetting = 0                                                        
        descrom[DESC_FSCFG_ADDR + 62 + 4] <= 8'h02;// bNumEndpoints = 2                                                            
        descrom[DESC_FSCFG_ADDR + 62 + 5] <= 8'hFF;// bInterfaceClass = CDC                                                        
        descrom[DESC_FSCFG_ADDR + 62 + 6] <= 8'hFF;// bInterfaceSubClass = none                                                    
        descrom[DESC_FSCFG_ADDR + 62 + 7] <= 8'hFF;// bInterafceProtocol = none                                                    
        descrom[DESC_FSCFG_ADDR + 62 + 8] <= 8'h05;// iInterface = none                                                            
        //----------------- Endpoint Descriptor -----------------                                                                  
        descrom[DESC_FSCFG_ADDR + 71 + 0] <= 8'h07;// bLength = 9 bytes                                                            
        descrom[DESC_FSCFG_ADDR + 71 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor                                        
        descrom[DESC_FSCFG_ADDR + 71 + 2] <= 8'h84;// bEndpointAddress = INPUT 4                                                   
        descrom[DESC_FSCFG_ADDR + 71 + 3] <= 8'h02;// TransferType = Bulk  
        descrom[DESC_FSCFG_ADDR + 71 + 4] <= 8'h00;                                                                                
        descrom[DESC_FSCFG_ADDR + 71 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                                                   
        descrom[DESC_FSCFG_ADDR + 71 + 6] <= 8'h00;// bInterval = 0 ms                                                             
        //----------------- Endpoint Descriptor -----------------                                                                  
        descrom[DESC_FSCFG_ADDR + 78 + 0] <= 8'h07;// bLength = 7 bytes                                                            
        descrom[DESC_FSCFG_ADDR + 78 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor                                        
        descrom[DESC_FSCFG_ADDR + 78 + 2] <= 8'h04;// bEndpointAddress = OUTPUT 4                                                  
        descrom[DESC_FSCFG_ADDR + 78 + 3] <= 8'h02;// TransferType = Bulk                                                          
        descrom[DESC_FSCFG_ADDR + 78 + 4] <= 8'h00;                                                                                
        descrom[DESC_FSCFG_ADDR + 78 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes                                                   
        descrom[DESC_FSCFG_ADDR + 78 + 6] <= 8'h00;// bInterval = 0 ms                                                             

        //------------Channel 3 I2C interface------------------
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        descrom[DESC_FSCFG_ADDR + 85 + 0] <= 8'h09;// bLength = 9 bytes                     
        descrom[DESC_FSCFG_ADDR + 85 + 1] <= 8'h04;// bDescriptorType = interface descriptor
        descrom[DESC_FSCFG_ADDR + 85 + 2] <= 8'h03;// bInterfaceNumber = 3                  
        descrom[DESC_FSCFG_ADDR + 85 + 3] <= 8'h00;// bAlternateSetting = 0                 
        descrom[DESC_FSCFG_ADDR + 85 + 4] <= 8'h02;// bNumEndpoints = 2                     
        descrom[DESC_FSCFG_ADDR + 85 + 5] <= 8'hFF;// bInterfaceClass = CDC                 
        descrom[DESC_FSCFG_ADDR + 85 + 6] <= 8'hFF;// bInterfaceSubClass = none             
        descrom[DESC_FSCFG_ADDR + 85 + 7] <= 8'hFF;// bInterafceProtocol = none             
        descrom[DESC_FSCFG_ADDR + 85 + 8] <= 8'h06;// iInterface = none                     
        //----------------- Endpoint Descriptor -----------------                           
        descrom[DESC_FSCFG_ADDR + 94 + 0] <= 8'h07;// bLength = 9 bytes                     
        descrom[DESC_FSCFG_ADDR + 94 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor 
        descrom[DESC_FSCFG_ADDR + 94 + 2] <= 8'h85;// bEndpointAddress = INPUT 5            
        descrom[DESC_FSCFG_ADDR + 94 + 3] <= 8'h02;// TransferType = Bulk                   
        descrom[DESC_FSCFG_ADDR + 94 + 4] <= 8'h00;                                         
        descrom[DESC_FSCFG_ADDR + 94 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes            
        descrom[DESC_FSCFG_ADDR + 94 + 6] <= 8'h00;// bInterval = 0 ms                      
        //----------------- Endpoint Descriptor -----------------                           
        descrom[DESC_FSCFG_ADDR + 101 + 0] <= 8'h07;// bLength = 7 bytes                    
        descrom[DESC_FSCFG_ADDR + 101 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 101 + 2] <= 8'h05;// bEndpointAddress = OUTPUT 5          
        descrom[DESC_FSCFG_ADDR + 101 + 3] <= 8'h02;// TransferType = Bulk                  
        descrom[DESC_FSCFG_ADDR + 101 + 4] <= 8'h00;                                        
        descrom[DESC_FSCFG_ADDR + 101 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes           
        descrom[DESC_FSCFG_ADDR + 101 + 6] <= 8'h00;// bInterval = 0 ms    
*/

                 
/*
        //------------Channel 4 PARALLEL interface------------------
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        descrom[DESC_FSCFG_ADDR + 108 + 0] <= 8'h09;// bLength = 9 bytes                       
        descrom[DESC_FSCFG_ADDR + 108 + 1] <= 8'h04;// bDescriptorType = interface descriptor  
        descrom[DESC_FSCFG_ADDR + 108 + 2] <= 8'h04;// bInterfaceNumber = 4                    
        descrom[DESC_FSCFG_ADDR + 108 + 3] <= 8'h00;// bAlternateSetting = 0                   
        descrom[DESC_FSCFG_ADDR + 108 + 4] <= 8'h02;// bNumEndpoints = 2                       
        descrom[DESC_FSCFG_ADDR + 108 + 5] <= 8'hFF;// bInterfaceClass = CDC                   
        descrom[DESC_FSCFG_ADDR + 108 + 6] <= 8'hFF;// bInterfaceSubClass = none               
        descrom[DESC_FSCFG_ADDR + 108 + 7] <= 8'hFF;// bInterafceProtocol = none               
        descrom[DESC_FSCFG_ADDR + 108 + 8] <= 8'h02;// iInterface = none                       
//        ----------------- Endpoint Descriptor -----------------                              
        descrom[DESC_FSCFG_ADDR + 117 + 0] <= 8'h07;// bLength = 7 bytes                       
        descrom[DESC_FSCFG_ADDR + 117 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor   
        descrom[DESC_FSCFG_ADDR + 117 + 2] <= 8'h86;// bEndpointAddress = INPUT 6              
        descrom[DESC_FSCFG_ADDR + 117 + 3] <= 8'h02;// TransferType = Bulk                     
        descrom[DESC_FSCFG_ADDR + 117 + 4] <= 8'h00;                                           
        descrom[DESC_FSCFG_ADDR + 117 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes              
        descrom[DESC_FSCFG_ADDR + 117 + 6] <= 8'h00;// bInterval = 0 ms                        
//        ----------------- Endpoint Descriptor -----------------                              
        descrom[DESC_FSCFG_ADDR + 124 + 0] <= 8'h07;// bLength = 7 bytes                       
        descrom[DESC_FSCFG_ADDR + 124 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor   
        descrom[DESC_FSCFG_ADDR + 124 + 2] <= 8'h06;// bEndpointAddress = OUTPUT 6             
        descrom[DESC_FSCFG_ADDR + 124 + 3] <= 8'h02;// TransferType = Bulk                     
        descrom[DESC_FSCFG_ADDR + 124 + 4] <= 8'h00;                                           
        descrom[DESC_FSCFG_ADDR + 124 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes              
        descrom[DESC_FSCFG_ADDR + 124 + 6] <= 8'h00;// bInterval = 0 ms                        
            

        //------------Serial COM 2 uart3 interface------------------
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        // bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02
        descrom[DESC_FSCFG_ADDR + 69 + 0] <= 8'h09;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 69 + 1] <= 8'h04;// bDescriptorType = interface descriptor
        descrom[DESC_FSCFG_ADDR + 69 + 2] <= 8'h02;// bInterfaceNumber = 2
        descrom[DESC_FSCFG_ADDR + 69 + 3] <= 8'h00;// bAlternateSetting = 0
        descrom[DESC_FSCFG_ADDR + 69 + 4] <= 8'h03;// bNumEndpoints = 2
        descrom[DESC_FSCFG_ADDR + 69 + 5] <= 8'h02;// bInterfaceClass = CDC
        descrom[DESC_FSCFG_ADDR + 69 + 6] <= 8'h01;// bInterfaceSubClass = none
        descrom[DESC_FSCFG_ADDR + 69 + 7] <= 8'h00;// bInterafceProtocol = none
        descrom[DESC_FSCFG_ADDR + 69 + 8] <= 8'h02;// iInterface = none
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 78 + 0] <= 8'h07;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 78 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 78 + 2] <= 8'h84;// bEndpointAddress = INPUT 4
        descrom[DESC_FSCFG_ADDR + 78 + 3] <= 8'h02;// bmAttributes = TransferType=Isochronous  SyncType=Adaptive  EndpointType=Data
        descrom[DESC_FSCFG_ADDR + 78 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 78 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 78 + 6] <= 8'h00;// bInterval = 0 ms
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 85 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 85 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 85 + 2] <= 8'h04;// bEndpointAddress = OUTPUT 4
        descrom[DESC_FSCFG_ADDR + 85 + 3] <= 8'h02;// TransferType = Bulk
        descrom[DESC_FSCFG_ADDR + 85 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 85 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 85 + 6] <= 8'h00;// bInterval = 0 ms
		//----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 92 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 92 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 92 + 2] <= 8'h8B;// // bEndpointAddress = INPUT B
        descrom[DESC_FSCFG_ADDR + 92 + 3] <= 8'h03;// TransferType = Interrupr
        descrom[DESC_FSCFG_ADDR + 92 + 4] <= 8'h08;
        descrom[DESC_FSCFG_ADDR + 92 + 5] <= 8'h00;// wMaxPacketSize = 8 bytes
        descrom[DESC_FSCFG_ADDR + 92 + 6] <= 8'h08;// bInterval = 0 ms
		
		
        //------------Serial COM 3 uart4 interface------------------
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        // bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02
        descrom[DESC_FSCFG_ADDR + 99 + 0] <= 8'h09;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 99 + 1] <= 8'h04;// bDescriptorType = interface descriptor
        descrom[DESC_FSCFG_ADDR + 99 + 2] <= 8'h03;// bInterfaceNumber = 3
        descrom[DESC_FSCFG_ADDR + 99 + 3] <= 8'h00;// bAlternateSetting = 0
        descrom[DESC_FSCFG_ADDR + 99 + 4] <= 8'h03;// bNumEndpoints = 3
        descrom[DESC_FSCFG_ADDR + 99 + 5] <= 8'h02;// bInterfaceClass = CDC
        descrom[DESC_FSCFG_ADDR + 99 + 6] <= 8'h01;// bInterfaceSubClass = none
        descrom[DESC_FSCFG_ADDR + 99 + 7] <= 8'h00;// bInterafceProtocol = none
        descrom[DESC_FSCFG_ADDR + 99 + 8] <= 8'h02;// iInterface = none
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 108 + 0] <= 8'h07;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 108 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 108 + 2] <= 8'h85;// bEndpointAddress = INPUT 5
        descrom[DESC_FSCFG_ADDR + 108 + 3] <= 8'h02;// bmAttributes = TransferType=Isochronous  SyncType=Adaptive  EndpointType=Data
        descrom[DESC_FSCFG_ADDR + 108 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 108 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 108 + 6] <= 8'h00;// bInterval = 0 ms
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 115 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 115 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 115 + 2] <= 8'h05;// bEndpointAddress = OUTPUT 5
        descrom[DESC_FSCFG_ADDR + 115 + 3] <= 8'h02;// TransferType = Bulk
        descrom[DESC_FSCFG_ADDR + 115 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 115 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 115 + 6]<= 8'h00;// bInterval = 0 ms
		//----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 122 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 122 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 122 + 2] <= 8'h8C;// bEndpointAddress = INPUT C
        descrom[DESC_FSCFG_ADDR + 122 + 3] <= 8'h03;// TransferType = Interrupr
        descrom[DESC_FSCFG_ADDR + 122 + 4] <= 8'h08;
        descrom[DESC_FSCFG_ADDR + 122 + 5] <= 8'h00;// wMaxPacketSize = 8 bytes
        descrom[DESC_FSCFG_ADDR + 122 + 6]<= 8'h08;// bInterval = 0 ms
		
		
        //------------Serial COM 4 uart5 interface------------------
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        // bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02
        descrom[DESC_FSCFG_ADDR + 129 + 0] <= 8'h09;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 129 + 1] <= 8'h04;// bDescriptorType = interface descriptor
        descrom[DESC_FSCFG_ADDR + 129 + 2] <= 8'h04;// bInterfaceNumber = 4
        descrom[DESC_FSCFG_ADDR + 129 + 3] <= 8'h00;// bAlternateSetting = 0
        descrom[DESC_FSCFG_ADDR + 129 + 4] <= 8'h03;// bNumEndpoints = 3
        descrom[DESC_FSCFG_ADDR + 129 + 5] <= 8'h02;// bInterfaceClass = CDC
        descrom[DESC_FSCFG_ADDR + 129 + 6] <= 8'h01;// bInterfaceSubClass = none
        descrom[DESC_FSCFG_ADDR + 129 + 7] <= 8'h00;// bInterafceProtocol = none
        descrom[DESC_FSCFG_ADDR + 129 + 8] <= 8'h02;// iInterface = none
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 138 + 0] <= 8'h07;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 138 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 138 + 2] <= 8'h86;// bEndpointAddress = INPUT 6
        descrom[DESC_FSCFG_ADDR + 138 + 3] <= 8'h02;// bmAttributes = TransferType=Isochronous  SyncType=Adaptive  EndpointType=Data
        descrom[DESC_FSCFG_ADDR + 138 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 138 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 138 + 6] <= 8'h00;// bInterval = 0 ms
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 145 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 145 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 145 + 2] <= 8'h06;// bEndpointAddress = OUTPUT 6
        descrom[DESC_FSCFG_ADDR + 145 + 3] <= 8'h02;// TransferType = Bulk
        descrom[DESC_FSCFG_ADDR + 145 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 145 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 145 + 6] <= 8'h00;// bInterval = 0 ms
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 152 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 152 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 152 + 2] <= 8'h8D;// bEndpointAddress = INPUT D
        descrom[DESC_FSCFG_ADDR + 152 + 3] <= 8'h03;// TransferType = Interrupr
        descrom[DESC_FSCFG_ADDR + 152 + 4] <= 8'h08;
        descrom[DESC_FSCFG_ADDR + 152 + 5] <= 8'h00;// wMaxPacketSize = 8 bytes
        descrom[DESC_FSCFG_ADDR + 152 + 6] <= 8'h08;// bInterval = 0 ms
		
		
        //------------Serial COM 5 uart6 interface------------------
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        // bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02
        descrom[DESC_FSCFG_ADDR + 159 + 0] <= 8'h09;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 159 + 1] <= 8'h04;// bDescriptorType = interface descriptor
        descrom[DESC_FSCFG_ADDR + 159 + 2] <= 8'h05;// bInterfaceNumber = 5
        descrom[DESC_FSCFG_ADDR + 159 + 3] <= 8'h00;// bAlternateSetting = 0
        descrom[DESC_FSCFG_ADDR + 159 + 4] <= 8'h03;// bNumEndpoints = 3
        descrom[DESC_FSCFG_ADDR + 159 + 5] <= 8'h02;// bInterfaceClass = CDC
        descrom[DESC_FSCFG_ADDR + 159 + 6] <= 8'h01;// bInterfaceSubClass = none
        descrom[DESC_FSCFG_ADDR + 159 + 7] <= 8'h00;// bInterafceProtocol = none
        descrom[DESC_FSCFG_ADDR + 159 + 8] <= 8'h02;// iInterface = none
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 168 + 0] <= 8'h07;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 168 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 168 + 2] <= 8'h87;// bEndpointAddress = INPUT 7
        descrom[DESC_FSCFG_ADDR + 168 + 3] <= 8'h02;// bmAttributes = TransferType=Isochronous  SyncType=Adaptive  EndpointType=Data
        descrom[DESC_FSCFG_ADDR + 168 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 168 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 168 + 6] <= 8'h00;// bInterval = 0 ms
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 175 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 175 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 175 + 2] <= 8'h07;// bEndpointAddress = OUTPUT 7
        descrom[DESC_FSCFG_ADDR + 175 + 3] <= 8'h02;// TransferType = Bulk
        descrom[DESC_FSCFG_ADDR + 175 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 175 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 175 + 6] <= 8'h00;// bInterval = 0 ms
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 182 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 182 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 182 + 2] <= 8'h8E;// bEndpointAddress = INPUT E
        descrom[DESC_FSCFG_ADDR + 182 + 3] <= 8'h03;// TransferType = Interrupt
        descrom[DESC_FSCFG_ADDR + 182 + 4] <= 8'h08;
        descrom[DESC_FSCFG_ADDR + 182 + 5] <= 8'h00;// wMaxPacketSize = 8 bytes
        descrom[DESC_FSCFG_ADDR + 182 + 6] <= 8'h08;// bInterval = 0 ms
		
		
        //------------Serial COM 6 uart7 interface------------------
        // 9 bytes standard interface AS descriptor(Interface 2, Alternate Setting 0):
        // bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02
        descrom[DESC_FSCFG_ADDR + 189 + 0] <= 8'h09;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 189 + 1] <= 8'h04;// bDescriptorType = interface descriptor
        descrom[DESC_FSCFG_ADDR + 189 + 2] <= 8'h06;// bInterfaceNumber = 6
        descrom[DESC_FSCFG_ADDR + 189 + 3] <= 8'h00;// bAlternateSetting = 0
        descrom[DESC_FSCFG_ADDR + 189 + 4] <= 8'h03;// bNumEndpoints = 3
        descrom[DESC_FSCFG_ADDR + 189 + 5] <= 8'h02;// bInterfaceClass = CDC
        descrom[DESC_FSCFG_ADDR + 189 + 6] <= 8'h01;// bInterfaceSubClass = none
        descrom[DESC_FSCFG_ADDR + 189 + 7] <= 8'h00;// bInterafceProtocol = none
        descrom[DESC_FSCFG_ADDR + 189 + 8] <= 8'h02;// iInterface = none
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 198 + 0] <= 8'h07;// bLength = 9 bytes
        descrom[DESC_FSCFG_ADDR + 198 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 198 + 2] <= 8'h88;// bEndpointAddress = INPUT 8
        descrom[DESC_FSCFG_ADDR + 198 + 3] <= 8'h02;// bmAttributes = TransferType=Isochronous  SyncType=Adaptive  EndpointType=Data
        descrom[DESC_FSCFG_ADDR + 198 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 198 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 198 + 6] <= 8'h00;// bInterval = 0 ms
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 205 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 205 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 205 + 2] <= 8'h08;// bEndpointAddress = OUTPUT 8
        descrom[DESC_FSCFG_ADDR + 205 + 3] <= 8'h02;// TransferType = Bulk
        descrom[DESC_FSCFG_ADDR + 205 + 4] <= 8'h00;
        descrom[DESC_FSCFG_ADDR + 205 + 5] <= 8'h02;// wMaxPacketSize = 512 bytes
        descrom[DESC_FSCFG_ADDR + 205 + 6] <= 8'h00;// bInterval = 0 ms
        //----------------- Endpoint Descriptor -----------------
        descrom[DESC_FSCFG_ADDR + 212 + 0] <= 8'h07;// bLength = 7 bytes
        descrom[DESC_FSCFG_ADDR + 212 + 1] <= 8'h05;// bDescriptorType = endpoint descriptor
        descrom[DESC_FSCFG_ADDR + 212 + 2] <= 8'h8F;// bEndpointAddress = INPUT F
        descrom[DESC_FSCFG_ADDR + 212 + 3] <= 8'h03;// TransferType = Interrupt
        descrom[DESC_FSCFG_ADDR + 212 + 4] <= 8'h08;
        descrom[DESC_FSCFG_ADDR + 212 + 5] <= 8'h00;// wMaxPacketSize = 8 bytes
        descrom[DESC_FSCFG_ADDR + 212 + 6] <= 8'h08;// bInterval = 1 ms
*/		



        if(descrom_len > DESC_STRLANG_ADDR)begin
            // string descriptor 0 (supported languages)
            descrom[DESC_STRLANG_ADDR + 0] <= 8'h04;// bLength = 4
            descrom[DESC_STRLANG_ADDR + 1] <= 8'h03;// bDescriptorType = string descriptor
            descrom[DESC_STRLANG_ADDR + 2] <= 8'h09;
            descrom[DESC_STRLANG_ADDR + 3] <= 8'h04;// wLangId[0] = 0x0409 = English U.S.
            descrom[DESC_STRVENDOR_ADDR + 0] <= 2 + 2*VENDORSTR_LEN;
            descrom[DESC_STRVENDOR_ADDR + 1] <= 8'h03;
            for(i = 0; i < VENDORSTR_LEN; i = i + 1) begin
                for(z = 0; z < 8; z = z + 1) begin
                    descrom[DESC_STRVENDOR_ADDR+ 2*i + 2][z] <= VENDORSTR[(VENDORSTR_LEN - 1 -i)*8+z];
                end
                descrom[DESC_STRVENDOR_ADDR+ 2*i + 3] <= 8'h00;
            end
            descrom[DESC_STRPRODUCT_ADDR + 0] <= 2 + 2*PRODUCTSTR_LEN;
            descrom[DESC_STRPRODUCT_ADDR + 1] <= 8'h03;
            for(i = 0; i < PRODUCTSTR_LEN; i = i + 1) begin
                for(z = 0; z < 8; z = z + 1) begin
                    descrom[DESC_STRPRODUCT_ADDR + 2*i + 2][z] <= PRODUCTSTR[(PRODUCTSTR_LEN - 1 - i)*8+z];
                end
                descrom[DESC_STRPRODUCT_ADDR + 2*i + 3] <= 8'h00;
            end

            descrom[DESC_STR3_ADDR + 0] <= 2 + 2*STR3_LEN;
            descrom[DESC_STR3_ADDR + 1] <= 8'h03;
            for(i = 0; i < STR3_LEN; i = i + 1) begin
                for(z = 0; z < 8; z = z + 1) begin
                    descrom[DESC_STR3_ADDR + 2*i + 2][z] <= STR3[(STR3_LEN - 1 - i)*8+z];
                end
                descrom[DESC_STR3_ADDR + 2*i + 3] <= 8'h00;
            end

//            descrom[DESC_STR4_ADDR + 0] <= 2 + 2*STR4_LEN;
//            descrom[DESC_STR4_ADDR + 1] <= 8'h03;
//            for(i = 0; i < STR4_LEN; i = i + 1) begin
//                for(z = 0; z < 8; z = z + 1) begin
//                    descrom[DESC_STR4_ADDR + 2*i + 2][z] <= STR4[(STR4_LEN - 1 - i)*8+z];
//                end
//                descrom[DESC_STR4_ADDR + 2*i + 3] <= 8'h00;
//            end

//            descrom[DESC_STR5_ADDR + 0] <= 2 + 2*STR5_LEN;
//            descrom[DESC_STR5_ADDR + 1] <= 8'h03;
//            for(i = 0; i < STR5_LEN; i = i + 1) begin
//                for(z = 0; z < 8; z = z + 1) begin
//                    descrom[DESC_STR5_ADDR + 2*i + 2][z] <= STR5[(STR5_LEN - 1 - i)*8+z];
//                end
//                descrom[DESC_STR5_ADDR + 2*i + 3] <= 8'h00;
//            end

//            descrom[DESC_STR6_ADDR + 0] <= 2 + 2*STR6_LEN;
//            descrom[DESC_STR6_ADDR + 1] <= 8'h03;
//            for(i = 0; i < STR6_LEN; i = i + 1) begin
//                for(z = 0; z < 8; z = z + 1) begin
//                    descrom[DESC_STR6_ADDR + 2*i + 2][z] <= STR6[(STR6_LEN - 1 - i)*8+z];
//                end
//                descrom[DESC_STR6_ADDR + 2*i + 3] <= 8'h00;
//            end


            descrom[DESC_STRSERIAL_ADDR + 0] <= 2 + 2*SERIALSTR_LEN;
            descrom[DESC_STRSERIAL_ADDR + 1] <= 8'h03;
            for(i = 0; i < SERIALSTR_LEN; i = i + 1) begin
                for(z = 0; z < 8; z = z + 1) begin
                    descrom[DESC_STRSERIAL_ADDR + 2*i + 2][z] <= SERIALSTR[(SERIALSTR_LEN - 1 - i)*8+z];
                end
                descrom[DESC_STRSERIAL_ADDR + 2*i + 3] <= 8'h00;
            end
        end
      end
    assign o_descrom_rdat = descrom[i_descrom_raddr];
endmodule
