/*
 * *****************************************************************************************
 *
 * 		Copyright (C) 2014-2023 Gowin Semiconductor Technology Co.,Ltd.
 *
 * @file		channels.h
 * @author		Embedded System R&D Department
 * @version		V1.0.0
 * @date		2023-05-30 09:00:00
 * @brief		Channels definitions.
 ******************************************************************************************
 */

#ifndef CHANNELS_H_
#define CHANNELS_H_


#if defined(__cplusplus)
extern "C" {
#endif


//Endpoint definitions

//UART
#define UART_USB_EP_IN         0x82        //Endpoint IN
#define UART_USB_EP_OUT        0x02        //Endpoint OUT

//JTAG
#define JTAG_USB_EP_IN         0x83        //Endpoint IN
#define JTAG_USB_EP_OUT        0x03        //Endpoint OUT

//SPI
#define SPI_USB_EP_IN          0x84        //Endpoint IN
#define SPI_USB_EP_OUT         0x04        //Endpoint OUT

//I2C
#define I2C_USB_EP_IN          0x85        //Endpoint IN
#define I2C_USB_EP_OUT         0x05        //Endpoint OUT

//Interface definitions
#define Interface_number		1			//Channel interface



//ID definitions
#define VENDOR_ID				0x33AA		//Vendor ID
#define PRODUCT_ID				0x0021		//Product ID


//Error code
#define ERR_COUNT 30
enum gwusb_error_code
{
    //General error
	SUCCESS                    =  0,        /** Success (no error) */
	USB_ERROR_IO               = -1,        /** Input/output error */
	USB_ERROR_INVALID_PARAM    = -2,        /** Invalid parameter */
	USB_ERROR_ACCESS           = -3,        /** Access denied (insufficient permissions) */
	USB_ERROR_NO_DEVICE        = -4,        /** No such device (it may have been disconnected) */
	USB_ERROR_NOT_FOUND        = -5,        /** Entity not found */
	USB_ERROR_BUSY             = -6,        /** Resource busy */
	USB_ERROR_TIMEOUT          = -7,        /** Operation timed out */
	USB_ERROR_OVERFLOW         = -8,        /** Overflow */
	USB_ERROR_PIPE             = -9,        /** Pipe error */
	USB_ERROR_INTERRUPTED      = -10,       /** System call interrupted (perhaps due to signal) */
	USB_ERROR_NO_MEM           = -11,       /** Insufficient memory */
	USB_ERROR_NOT_SUPPORTED    = -12,       /** Operation not supported or unimplemented on this platform */
    //USB to IIC error
	USB2IIC_ERR_SENDCMD_FAIL   = -13,       /** USB to IIC send command fail*/
	USB2IIC_ERR_GETREG_FAIL    = -14,       /** USB to IIC get iic registers fail*/
	USB2IIC_ERR_SETREG_FAIL    = -15,       /** USB to IIC set iic registers fail*/
	USB2IIC_ERR_INVALID_PARAM  = -16,       /** USB to IIC invalid timing parameter*/
	USB2IIC_ERR_TXDATA_FAIL    = -17,       /** USB to IIC iic tx data fail*/
	USB2IIC_ERR_RXDATA_FAIL    = -18,       /** USB to IIC iic rx data fail*/
	USB2IIC_ERR_CMPL_TIMEOUT   = -19,       /** USB to IIC waiting complete timeout*/
	//USB to SPI error
	USB2SPI_ERR_SENDCMD_FAIL   = -20,       /** USB to SPI send command fail*/
	USB2SPI_ERR_GETREG_FAIL    = -21,       /** USB to SPI get registers fail*/
	USB2SPI_ERR_SETREG_FAIL    = -22,       /** USB to SPI set registers fail*/
	USB2SPI_ERR_INVALID_PARAM  = -23,       /** USB to SPI invalid timing parameter*/
	USB2SPI_ERR_TXDATA_FAIL    = -24,       /** USB to SPI tx data fail*/
	USB2SPI_ERR_RXDATA_FAIL    = -25,       /** USB to SPI rx data fail*/
	USB2SPI_ERR_CMPL_TIMEOUT   = -26,       /** USB to SPI waiting complete timeout*/

	U2J_ERROR_USBTRANS_ERR     = -27, /** usb2jtag transmit error*/
	U2J_ERROR_INVALID_PARAM    = -28, /** usb2jtag invalid parameter*/


	ERROR_OTHER = -99 /** Other error */
};


#if defined(__cplusplus)
}
#endif

#endif // CHANNELS_H_
