/*
 * *********************************************************************************************
 *
 * 		Copyright (C) 2014-2023 Gowin Semiconductor Technology Co.,Ltd.
 *
 * @file        gw_usb2jtag.h
 * @author      Embedded System R&D Department
 * @version     V1.0.0
 * @date        2023-03-23 09:00:00
 * @brief       GWUSB2XX JTAG API
 ***********************************************************************************************
 */

#ifndef GW_USB2JTAG_H_
#define GW_USB2JTAG_H_


//Cross platform
#if defined(_WIN32) || defined(__CYGWIN__)
#ifndef DLLIMPORT
#define DLLIMPORT __declspec(dllexport)
#endif
#else
#define DLLIMPORT
#endif


/* Definitions ---------------------------------------------------------------------- */

#define TMS_TDI_FEDGE             0x4B
#define TMS_TDI_REDGE             0x4A
#define TMS_TDI_TDO               0x6E

#define DATA_TDI_BYTES_MSB        0x11
#define DATA_TDI_BITS_MSB         0x13
#define DATA_TDO_BYTES_MSB        0x20

#define DATA_TDI_BITS_LSB         0x1B
#define DATA_TDI_BYTES_LSB        0x19
#define DATA_TDO_BITS_LSB         0x2A
#define DATA_TDO_BYTES_LSB        0x28



//flash

//#define W25Q64JV   0xEF4017
#define W25Q64JV   0x1740ef


#define  Select_Tck 0x0008// Set frequency :30MHZ/(Select_Tck*2)



#if defined(__cplusplus)
extern "C" {
#endif


/* Declarations ---------------------------------------------------------------------- */
DLLIMPORT void usb2jtag_read(libusb_device_handle *devh,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_read_back(libusb_device_handle *devh,unsigned int uiTimeout);

DLLIMPORT int usb2jtag_tck_select(libusb_device_handle *devh,unsigned int select_tck,unsigned int uiTimeout);

DLLIMPORT int usb2jtag_tap_reset(libusb_device_handle *devh, unsigned int uiTimeout);
DLLIMPORT int usb2jtag_from_rst_to_idle(libusb_device_handle *devh, unsigned int uiTimeout);
DLLIMPORT int usb2jtag_shift_ir(libusb_device_handle *devh, unsigned int uiTimeout);
DLLIMPORT int usb2jtag_shift_dr(libusb_device_handle *devh, unsigned int uiTimeout);
DLLIMPORT int usb2jtag_ConfigData_Send (libusb_device_handle *devh,unsigned char *Usb2JtagCmd,unsigned int uiTimeout);

DLLIMPORT int usb2jtag_Read_IDCODE(libusb_device_handle *devh,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_Read_Status_Register(libusb_device_handle *devh,unsigned int uiTimeout);

DLLIMPORT int usb2jtag_Erase_Sram(libusb_device_handle *devh,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_BitstreamData_send(libusb_device_handle *devh,const char *filename,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_Config_Sram(libusb_device_handle *devh,const char *filename,unsigned int uiTimeout);

DLLIMPORT void usb2jtag_TckSet_Delay(libusb_device_handle *devh,unsigned int select_tck, int uiTimeout);
void jtag_ef_prog_one_X_TSMC(libusb_device_handle *devh,unsigned int uiTimeout,uint8_t data_array[256], uint32_t address_index);
DLLIMPORT int usb2jtag_Flash_Program(libusb_device_handle *devh,const char *filename,int verify ,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_embed_Program_Flash(libusb_device_handle *devh,const char *filename,int verify ,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_Erase_T_Flash(libusb_device_handle *devh,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_read_emb_Flash(libusb_device_handle *devh,const char *filename,unsigned int uiTimeout);

DLLIMPORT void usb2jtag_read_SPI_Flash_id_onetime(libusb_device_handle *devh,int cnt,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_read_SPI_Flash_id(libusb_device_handle *devh,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_Erase_SPI_Flash(libusb_device_handle *devh,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_read_SPI_Flash(libusb_device_handle *devh,const char *filename,unsigned int uiTimeout);
DLLIMPORT void usb2jtag_Program_SPI_Flash_page(libusb_device_handle *devh,unsigned char *data_array,int addr,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_Program_SPI_Flash(libusb_device_handle *devh,const char *filename,unsigned int uiTimeout);
DLLIMPORT int usb2jtag_Extern_Program_Flash(libusb_device_handle *devh,const char *filename,unsigned int uiTimeout);




#if defined(__cplusplus)
}
#endif

#endif   //#ifndef GW_USB2JTAG_H_
