/*
 * *********************************************************************************************
 *
 * 		Copyright (C) 2014-2023 Gowin Semiconductor Technology Co.,Ltd.
 *
 * @file        gw_usb2uart_vcp_asyn.c
 * @author      Embedded System R&D Department
 * @version     V2.0.0
 * @date        2023-03-23 09:00:00
 * @brief       Asynchronous USB2UART virtual com port
 ***********************************************************************************************
 */

/* Includes ---------------------------------------------------------------------- */
#include <stdio.h>
#include <windows.h>

#include "gw_usb2uart.h"

/* Declarations ---------------------------------------------------------------------- */
static DWORD WINAPI ThreadRxMsg(LPVOID lpParameter);
static void getComName(char* ComNamePtr, unsigned int ComNum);
static void printBuf(char *buf, unsigned int bufSize);


/* Declarations ---------------------------------------------------------------------- */
char ComName[13];
HANDLE hCom_1;
unsigned int readcount = 0;

char DataBuffer[] = "UART send data test";


/* Functions ---------------------------------------------------------------------- */
int usb2uart_vcp_async(void)
{
	DWORD dwError;
	unsigned int ComNum = 0;

    ComNum = UART_COM_PORT;


	getComName(ComName, ComNum);
    printf("ComName %s\r\n",ComName);
    printf("ComName %d\r\n",ComNum);
    // Open comm Port
	hCom_1 =
	CreateFile (
		ComName,
		GENERIC_READ | GENERIC_WRITE,
		0,
		NULL,
		OPEN_EXISTING,
		FILE_FLAG_OVERLAPPED,       //Asynchronous
		0
	);

	if(hCom_1 == INVALID_HANDLE_VALUE)
	{
	    int a = GetLastError();
	    printf("Error : %d\n", a);

	    return -1;
	}

	// Set read timeout
	COMMTIMEOUTS timeouts;
	GetCommTimeouts(hCom_1, &timeouts);
	timeouts.ReadIntervalTimeout         = 0;
	timeouts.ReadTotalTimeoutMultiplier  = 0;
	timeouts.ReadTotalTimeoutConstant    = 6000;
	timeouts.WriteTotalTimeoutMultiplier = 0;
	timeouts.WriteTotalTimeoutConstant   = 0;
	SetCommTimeouts(hCom_1, &timeouts);

	// Set read/write buffer size
	static const int nBufSize = 32768;
	if(!SetupComm(hCom_1, nBufSize, nBufSize))
	{
		printf("SetupComm() failed, comm port closed...\n");
		CloseHandle(hCom_1);

		return -1;
	}

	// set comm port configurations
	DCB dcb;

	if (!GetCommState(hCom_1, &dcb))
	{
	    printf("GetCommState() failed, comm port closed...\n");
	    CloseHandle(hCom_1);

	    return -1;
	}

	//configurations setting
	int nBaud = 115200;           //baud rate
	dcb.DCBlength = sizeof(DCB);
	dcb.BaudRate  = nBaud;        //baud rate
	dcb.Parity    = NOPARITY;     //parity
	dcb.ByteSize  = 8;            //data size
	dcb.StopBits  = ONESTOPBIT;   //stop bits

	if (!SetCommState(hCom_1, &dcb))
	{
		printf("SetCommState() failed\n");
	    CloseHandle(hCom_1);

	    return -1;
	}

	OVERLAPPED wrOverlapped;

	memset(&wrOverlapped, 0, sizeof(wrOverlapped));

	if (wrOverlapped.hEvent != NULL)
	{
		ResetEvent(wrOverlapped.hEvent);
		wrOverlapped.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	}

    DWORD dwBytesToWrite = (DWORD)strlen(DataBuffer);
    DWORD dwBytesWritten = 0;

	if (ClearCommError(hCom_1, &dwError, NULL))
	{
		PurgeComm(hCom_1, PURGE_TXABORT | PURGE_TXCLEAR);
	}

	if (!WriteFile(hCom_1, DataBuffer, dwBytesToWrite, &dwBytesWritten, &wrOverlapped))
	{
		if (GetLastError() == ERROR_IO_PENDING)
		{
			while (!GetOverlappedResult(hCom_1, &wrOverlapped, &dwBytesWritten, FALSE))
			{
				if (GetLastError() == ERROR_IO_INCOMPLETE)
				{
					//Not finished, continue and get the result again
					continue;
				}
				else
				{
					printf("Send Error Occured\n");
					ClearCommError(hCom_1, &dwError,NULL);

					break;
				}
			}

			printf("Data send finished, Bytes Written: %lu\n", dwBytesWritten);
		}
	}

	//clear buffer
	PurgeComm(hCom_1, PURGE_RXCLEAR | PURGE_RXABORT);

	//clear error
	COMSTAT cs;

	if(!ClearCommError(hCom_1, &dwError, &cs))
	{
		printf("ClearCommError() failed\n");
		CloseHandle(hCom_1);

		return -1;
	}

	SetCommMask(hCom_1, EV_RXCHAR);
	HANDLE hThread1 = CreateThread(NULL, 0, ThreadRxMsg, NULL, 0, NULL);
	if(hThread1 == NULL)
    {
        printf("Create thread failed.\n");
        CloseHandle(hCom_1);

        return -1;
    }

	while(1);

	CloseHandle(hCom_1);

	return 0;
}


//Receive data asynchronously
static DWORD WINAPI ThreadRxMsg(LPVOID lpParameter)
{
	while(1)
	{
		OVERLAPPED osWait;

		memset(&osWait, 0, sizeof(OVERLAPPED));

		osWait.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

		DWORD dwEvtMask;

		if (WaitCommEvent(hCom_1, &dwEvtMask, &osWait))
		{
			if(dwEvtMask & EV_RXCHAR)
			{
				DWORD dwError;
				COMSTAT cs;
				if(!ClearCommError(hCom_1, &dwError, &cs))
				{
					printf("ClearCommError() failed\n");
					CloseHandle(hCom_1);

					return -1;
				}

				char buf[101] = {0};
				DWORD nLenOut = 0;
				OVERLAPPED osRead;

				memset(&osRead, 0, sizeof(OVERLAPPED));

				osRead.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

				BOOL bReadStatus = ReadFile(hCom_1, buf, cs.cbInQue, &nLenOut, &osRead);

				if(!bReadStatus)
				{
					if(GetLastError() == ERROR_IO_PENDING)
					{
						printf("GetLastError() == ERROR_IO_PENDING\n");
						// to do
					}
				}
				else
				{
					printBuf(buf, nLenOut);
				}
			}
		}  // if (WaitCommEvent(hCom_1, &dwEvtMask, &osWait))
		else
		{
			if(GetLastError() == ERROR_IO_PENDING)
			{
				WaitForSingleObject(osWait.hEvent, INFINITE);

				if(dwEvtMask & EV_RXCHAR)
				{
					DWORD dwError;
					COMSTAT cs;

					if(!ClearCommError(hCom_1, &dwError, &cs))
					{
						printf("ClearCommError() failed\n");
						CloseHandle(hCom_1);

						return -1;
					}

					char buf[101] = {0};
					DWORD nLenOut = 0;
					OVERLAPPED osRead;

					memset(&osRead, 0, sizeof(OVERLAPPED));

					osRead.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

					BOOL bReadStatus = ReadFile(hCom_1, buf, cs.cbInQue, &nLenOut, &osRead);

					if(!bReadStatus)
					{
						if(GetLastError() == ERROR_IO_PENDING)
						{
							printf("GetLastError() == ERROR_IO_PENDING\n");
							// to do
						}
					}
					else{
						printBuf(buf, nLenOut);
					}
				}
			}
		}
	}

	return 1;
}


static void getComName(char* ComNamePtr, unsigned int ComNum)
{
	if(ComNum > 9)
    {
		sprintf(ComNamePtr, "\\\\.\\COM%d", ComNum);
    }
	else
    {
		sprintf(ComNamePtr, "COM%d", ComNum);
    }
}


static void printBuf(char *buf, unsigned int bufSize)
{
	for(int i = 0; i < bufSize; i++)
	{
#ifdef PRINT_HEX
		printf("%02x", buf[i]);
#else
		printf("%c", buf[i]);
#endif

#ifdef SEPARATE
		if((readcount + i + 1) % 32 == 0)
        {
            printf("\n");
        }
		else if((readcount + i + 1) % 8 == 0)
        {
            printf("   ");
        }
		else
        {
            printf(" ");
        }
#else
//		printf(" ");
#endif
	}

	readcount += bufSize;
}
