`timescale 1ns / 100ps
module USB_to_Multi_Serial_tb;
//Ports
reg       phy_clk           = 1       ;
reg       rst_n             = 0       ;
//uart ports  
wire        UART_TXD                  ;
reg         UART_RXD        = 1       ;
reg         UART_CTS        = 0       ;
reg [31:0]  BAUD_RATE       = 115200  ;
reg [7:0]   PARITY_BIT      = 0       ;
reg [7:0]   STOP_BIT        = 0       ;
reg [7:0]   DATA_BITS       = 8'h8    ;
reg [15:0]  TX_DATA         = 0       ;
reg         TX_DATA_VAL     = 0       ;
wire        TX_BUSY                   ;
wire [15:0] RX_DATA                   ;
wire        RX_DATA_VAL               ;
//jtag ports
reg  usb_jtag_tck           = 1       ;
reg  usb_jtag_rx_dval       = 0       ;
reg [ 7:0] usb_jtag_rx_data = 0       ;
wire  usb_jtag_in_rdy                 ;
wire  usb_jtag_tx_dval                ;
wire [ 7:0] usb_jtag_tx_data          ;
wire  jtag_clk                        ;
wire  jtag_tdi                        ;
reg  jtag_tdo               = 1       ;
wire  jtag_tms                        ;
//spi ports   
reg  usb_spi_clk            = 0       ;
reg  usb_spi_rx_dval        = 0       ;
reg [ 7:0] usb_spi_rx_data  = 0       ;
wire  usb_spi_tx_dval                 ;
wire [ 7:0] usb_spi_tx_data           ;
reg  usb_spi_rx_dval_en     = 0       ;
wire  spi_clk_o                       ;
wire  spi_cs_n_o                      ;
wire  spi_miso_io                     ;
wire  spi_mosi_io                     ;
reg  spi_miso_io_en         = 0       ;
reg clk_out                 = 0       ;
//i2c ports 
reg  usb_i2c_clk            = 0       ;
reg  usb_i2c_rx_dval_en     = 0       ;
reg  usb_i2c_rx_dval        = 0       ;
reg [7:0] usb_i2c_rx_data   = 0       ;
wire  usb_i2c_tx_dval                 ;
wire [7:0] usb_i2c_tx_data            ;
wire  I2C1_SCL                        ;
wire  I2C1_SDA                        ;

USB_to_Multi_Serial_Top  USB_to_Multi_Serial_Top_u (
  .phy_clk(phy_clk)         ,
  .rst_n(rst_n)             ,
  .UART_TXD(UART_TXD)       ,
  .UART_RXD(UART_RXD)       ,
  .UART_RTS()               ,
  .UART_CTS(UART_CTS)       ,
  .BAUD_RATE(BAUD_RATE)     ,
  .PARITY_BIT(PARITY_BIT)   ,
  .STOP_BIT(STOP_BIT)       ,
  .DATA_BITS(DATA_BITS)     ,
  .TX_DATA(TX_DATA)         ,
  .TX_DATA_VAL(TX_DATA_VAL) ,
  .TX_BUSY(TX_BUSY)         ,
  .RX_DATA(RX_DATA)         ,
  .RX_DATA_VAL(RX_DATA_VAL) ,

  .usb_jtag_tck(usb_jtag_tck)        ,
  .usb_jtag_rx_dval(usb_jtag_rx_dval),
  .usb_jtag_rx_data(usb_jtag_rx_data),
  .usb_jtag_in_rdy(usb_jtag_in_rdy)  ,
  .usb_jtag_tx_dval(usb_jtag_tx_dval),
  .usb_jtag_tx_data(usb_jtag_tx_data),
  .jtag_clk(jtag_clk)                ,
  .jtag_tms(jtag_tms)                ,
  .jtag_tdo(jtag_tdo)                ,
  .jtag_tdi(jtag_tdi)                ,

  .usb_spi_clk(usb_spi_clk)         ,
  .usb_spi_rx_dval(usb_spi_rx_dval) ,
  .usb_spi_rx_data(usb_spi_rx_data) ,
  .usb_spi_tx_dval(usb_spi_tx_dval) ,
  .usb_spi_tx_data(usb_spi_tx_data) ,
  .spi_clk_o(spi_clk_o)             ,
  .spi_cs_n_o(spi_cs_n_o)           ,
  .spi_miso_io(spi_miso_io)         ,
  .spi_mosi_io(spi_mosi_io)         ,

  .usb_i2c_clk(usb_i2c_clk),
  .usb_i2c_rx_dval(usb_i2c_rx_dval) ,
  .usb_i2c_rx_data(usb_i2c_rx_data) ,
  .usb_i2c_tx_dval(usb_i2c_tx_dval) ,
  .usb_i2c_tx_data(usb_i2c_tx_data) ,
  .I2C1_SCL(I2C1_SCL)               ,
  .I2C1_SDA(I2C1_SDA)
);

//clk
always #5  phy_clk      = ! phy_clk      ;//uart clk
always #10 usb_jtag_tck = !usb_jtag_tck  ;//jtag phy_clk 
always #5  usb_spi_clk  = ! usb_spi_clk  ;//spi clk
always #5  usb_i2c_clk  = ! usb_i2c_clk  ;//i2c clk
//reast
initial begin 
  #10   rst_n = 1;
  #200  rst_n = 0; 
end

//UART_TX
initial begin
  #1000  TX_DATA_VAL = 1'b0;
  #100 
  #5  TX_DATA = 16'h55;  TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #46890  TX_DATA = 16'h41; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h54; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h52; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52100  TX_DATA = 16'h20; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h73; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h65; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h6E; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h64; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52100  TX_DATA = 16'h20; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h64; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h61; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h74; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h61; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52100  TX_DATA = 16'h20; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h74; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h65; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h73; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #52000  TX_DATA = 16'h74; TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
  #20 TX_DATA_VAL = 1'b1;
  #10 TX_DATA_VAL = 1'b0;
#50000 $finish;
end

//spi_tx
assign spi_miso_io = spi_miso_io_en;
always @ (posedge spi_clk_o )
    begin
      if (rst_n)
        clk_out <= 0;
      else 
        clk_out <= ~clk_out;
    end
always @(posedge usb_spi_clk)begin
  if(rst_n)
    usb_spi_rx_dval <= 1'b0;
  else if(usb_spi_rx_dval_en)
    usb_spi_rx_dval <= ~usb_spi_rx_dval;
  else
    usb_spi_rx_dval <= 1'b0;
end
initial begin 
  #400  
  #10   usb_spi_rx_dval_en = 1;
  #25   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h02;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h38;
  
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h40;
  #20   usb_spi_rx_data = 8'h40;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h80;
  #20   usb_spi_rx_data = 8'h07;
  #20   usb_spi_rx_data = 8'h03;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h09;
  #20   usb_spi_rx_data = 8'hf0;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h61;
  #20   usb_spi_rx_data = 8'h10;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h11;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'ha5;
  #20   usb_spi_rx_data = 8'hbf;
  #20   usb_spi_rx_data = 8'h21;
  #20   usb_spi_rx_data = 8'hb8;
  #20   usb_spi_rx_data = 8'h9e;
  #20   usb_spi_rx_data = 8'hec;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;
  #20   usb_spi_rx_data = 8'h00;

  #20   usb_spi_rx_data = 8'h02;
  #20   usb_spi_rx_data = 8'h00;
  #60   usb_spi_rx_data = 8'h11;
  #20   usb_spi_rx_data = 8'h22;
  #20   usb_spi_rx_data = 8'h33;
  #20   usb_spi_rx_data = 8'h44;  
  #20   usb_spi_rx_data = 8'h55;
  #20   usb_spi_rx_data = 8'h66;
  #20   usb_spi_rx_data = 8'h77;
  #20   usb_spi_rx_data = 8'h88;
  #20   usb_spi_rx_data = 8'h99;
  #20   usb_spi_rx_data = 8'hAA;
  #20   usb_spi_rx_data = 8'hBB;
  #20   usb_spi_rx_data = 8'hCC;
  #20   usb_spi_rx_data = 8'hDD;
  #20   usb_spi_rx_data = 8'hEE;
  #20   usb_spi_rx_data = 8'hFF;
  #20   usb_spi_rx_data = 8'h10;
  #20   usb_spi_rx_dval_en = 0;
end 
//integer w_file;
//    initial w_file = $fopen("data_out_miss.txt");
//    always @(clk_out)
//    begin 
//        $fdisplay(w_file,"%b",spi_mosi_io);   
//    end
//spi_rx
initial begin 
    #7000 
    #10   usb_spi_rx_dval_en = 1;
    #5   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h02;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h28;
    
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;

    #20   usb_spi_rx_data = 8'h40;
    #20   usb_spi_rx_data = 8'h40;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h80;
    #20   usb_spi_rx_data = 8'h07;
    #20   usb_spi_rx_data = 8'h03;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h09;
    #20   usb_spi_rx_data = 8'hf0;

    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h20;
    #20   usb_spi_rx_data = 8'h10;

    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h11;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;

    #20   usb_spi_rx_data = 8'h08;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;
  
    #20   usb_spi_rx_data = 8'h03;
    #20   usb_spi_rx_data = 8'h00;
    #60   usb_spi_rx_data = 8'h00;
    #20   usb_spi_rx_data = 8'h00;

    #20   usb_spi_rx_dval_en = 0;
    #100  spi_miso_io_en = 1;
    #40   spi_miso_io_en = 1;
    #20   spi_miso_io_en = 1;
    #40   spi_miso_io_en = 1;
    #20   spi_miso_io_en = 1;
    #200   spi_miso_io_en = 0;

    #58000
          usb_spi_rx_dval = 1;
          usb_spi_rx_data = 8'h01;
    #10   usb_spi_rx_dval = 0;

    #10   usb_spi_rx_dval = 1;
          usb_spi_rx_data = 8'h00;
    #10   usb_spi_rx_dval = 0;

    #10   usb_spi_rx_dval = 1;
          usb_spi_rx_data = 8'h02;
    #10   usb_spi_rx_dval = 0;

    #10   usb_spi_rx_dval = 1;
          usb_spi_rx_data = 8'h00;
    #10   usb_spi_rx_dval = 0;

    #10   usb_spi_rx_dval = 1;
    #10   usb_spi_rx_dval = 0;
    #10   usb_spi_rx_dval = 1;
    #10   usb_spi_rx_dval = 0;
    #10   usb_spi_rx_dval = 1;
    #10   usb_spi_rx_dval = 0;
    #10   usb_spi_rx_dval = 1;
    #10   usb_spi_rx_dval = 0;
end

//i2c_tx////////////////////////////////////////////////////////////////////
pullup p1(I2C1_SCL);
pullup p2(I2C1_SDA);
wire test_data_i2c;
wire I2C1_SCL_negedge;
reg i2c_cnt_en = 0;
reg [9:0]i2c_cnt;

always @(posedge usb_i2c_clk)begin//i2c usb_i2c_rx_dval en
  if(rst_n)
    usb_i2c_rx_dval <= 1'b0;
  else if(usb_i2c_rx_dval_en)
    usb_i2c_rx_dval <= ~usb_i2c_rx_dval;
  else
    usb_i2c_rx_dval <= 1'b0;
end

initial begin 
  #50000 
  //config  
  #10   usb_i2c_rx_dval_en = 1;
  #25   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h03;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h1c;

  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h01;
  #20   usb_i2c_rx_data = 8'h02;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h50;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h1e;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h65;
  #20   usb_i2c_rx_data = 8'h03;
  #20   usb_i2c_rx_data = 8'h65;
  #20   usb_i2c_rx_data = 8'h02;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_dval_en = 0;
  #5000
  //data sent
  #10   usb_i2c_rx_dval_en = 1;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h03;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h30;
  
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h01;
  #20   usb_i2c_rx_data = 8'h02;

  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h50;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h14;
  #20   usb_i2c_rx_data = 8'h1e;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h01;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h65;
  #20   usb_i2c_rx_data = 8'h03;
  #20   usb_i2c_rx_data = 8'h65;

  #20   usb_i2c_rx_data = 8'h02;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h01;
  #20   usb_i2c_rx_data = 8'h02;
  #20   usb_i2c_rx_data = 8'h03;
  #20   usb_i2c_rx_data = 8'h04;
  #20   usb_i2c_rx_data = 8'h05;
  #20   usb_i2c_rx_data = 8'h06;
  #20   usb_i2c_rx_data = 8'h07;
  #20   usb_i2c_rx_data = 8'h08;
  #20   usb_i2c_rx_data = 8'h09;
  #20   usb_i2c_rx_data = 8'h0a;
  #20   usb_i2c_rx_data = 8'h0b;
  #20   usb_i2c_rx_data = 8'h0c;
  #20   usb_i2c_rx_data = 8'h0d;
  #20   usb_i2c_rx_data = 8'h0e;
  #20   usb_i2c_rx_data = 8'h0f;
  #20   usb_i2c_rx_data = 8'h10;
  #20   usb_i2c_rx_data = 8'h11;
  #20   usb_i2c_rx_data = 8'h12;
  #20   usb_i2c_rx_data = 8'h13;
  #20   usb_i2c_rx_data = 8'h14;
  #20   usb_i2c_rx_dval_en = 0;
        i2c_cnt_en = 1;    
end 
assign I2C1_SCL_negedge = ~I2C1_SCL;//I2C1_SCL_negedge 
assign I2C1_SDA = (i2c_cnt>=10'd9)? 1'b0 : 1'bz;
assign test_data_i2c = I2C1_SDA & I2C1_SCL;
always @(negedge I2C1_SCL or posedge rst_n)begin
  if(rst_n)
    i2c_cnt <= 0;
  else if(i2c_cnt>=10'd9)
    i2c_cnt <= 1;
  else if(i2c_cnt_en)
    i2c_cnt <= i2c_cnt + 1;
  else
    i2c_cnt <= i2c_cnt;
end

//i2c_rx////////////////////////////////////////////////////////////////////
initial begin 
  #1500  
  #10   usb_i2c_rx_dval_en = 1;
        //assign I2C1_SCL =  i2c1_scl_o ? 1'b0 : 1'bz;
  #25   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h03;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h1c;
  
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h01;
  #20   usb_i2c_rx_data = 8'h02;

  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h50;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h1e;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h65;
  #20   usb_i2c_rx_data = 8'h03;
  #20   usb_i2c_rx_data = 8'h65;
  #20   usb_i2c_rx_data = 8'h02;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_dval_en = 0;

  #10000  
  #10   usb_i2c_rx_dval_en = 1;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h03;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h1c;
  
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h01;
  #20   usb_i2c_rx_data = 8'h02;

  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h50;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h09;
  #20   usb_i2c_rx_data = 8'h1f;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h01;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;

  #20   usb_i2c_rx_data = 8'h65;
  #20   usb_i2c_rx_data = 8'h03;
  #20   usb_i2c_rx_data = 8'h65;

  #20   usb_i2c_rx_data = 8'h02;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_data = 8'h00;
  #20   usb_i2c_rx_dval_en = 0;
end 

//JTAG_read_id_code （id：aa 02 1b 98 00 01）

initial begin
    #10000
    //4B 04 1F
        usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h4B;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h04;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h1f;
    #10 usb_jtag_rx_dval = 0;
  //4B 00,00
    #4800
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h4B;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h00;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h00;
    #10 usb_jtag_rx_dval = 0;
  //4B 03,03
    #2800
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h4B;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h03;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h03;
    #10 usb_jtag_rx_dval = 0;
  //1B 06 11
    #8000
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h1B;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h06;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h11;
    #10 usb_jtag_rx_dval = 0;
    #180 jtag_tdo = 0;
    #120 jtag_tdo = 1;
    #40  jtag_tdo = 0;
  //4B 00 01
    #4000
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h4B;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h00;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h01;
    #10 usb_jtag_rx_dval = 0;
    //4B 04 01
    #8000
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h4B;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h04;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h01;
    #10 usb_jtag_rx_dval = 0;
    #530 jtag_tdo = 1;
    //4B 02 01
    #4000
    #20 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h4B;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h02;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h01;
    #10 usb_jtag_rx_dval = 0;
  //28 02 00
    #8000
    #5 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h28;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h02;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h00;
    #10 usb_jtag_rx_dval = 0;
    #260 jtag_tdo = 0;
    #40  jtag_tdo = 1;
    #80  jtag_tdo = 0;
    #260 jtag_tdo = 1;
    #80  jtag_tdo = 0;
    #80  jtag_tdo = 1;
    #60  jtag_tdo = 0;
  //2A 06
    #4000
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h2a;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h06;
    #10 usb_jtag_rx_dval = 0;
    #160 jtag_tdo = 1;
    #40  jtag_tdo = 0;
  //6E 00 81
    #8000
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h6e;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h00;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h81;
    #10 usb_jtag_rx_dval = 0;
  //4B 01 81
    #4000
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h4b;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h01;
    #10 usb_jtag_rx_dval = 0;
  
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h81;
    #10 usb_jtag_rx_dval = 0;
    #430 jtag_tdo = 1;
  //87
    #8010
    #10 usb_jtag_rx_dval = 1;
        usb_jtag_rx_data = 8'h87;
    #10 usb_jtag_rx_dval = 0;

end


endmodule